import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import 'Data.dart';
import 'src/Constants.dart';
import 'src/PasswordScreen.dart';
import 'src/Requests/AcquireUserParams.dart';
import 'src/Requests/BusinessDetailParams.dart';
import 'src/Requests/OtpParams.dart';
import 'src/Responses/User.dart';
import 'src/Utilities/Utils.dart';
import 'src/VerifyUser.dart';

class BusinessSignUpForm extends StatefulWidget {
  final String apiKey;
  final Data predefinedData;
  final Map<String, String> data;
  final ValueSetter<String> callbackBusinessId;
  final User user;
  final loginFromEmail;
  BusinessSignUpForm({Key key, @required this.apiKey, this.data,
    this.predefinedData, this.callbackBusinessId, this.user,
    this.loginFromEmail}) : super(key: key);
  @override
  _BusinessSignUpFormState createState() =>
      _BusinessSignUpFormState(predefinedData, user);
}
class _BusinessSignUpFormState extends State<BusinessSignUpForm> {
  final _formKey = GlobalKey<FormState>();
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();
  final _emailController = TextEditingController();
  final _mobileController = TextEditingController();
  Widget logo;
  Color textColor;
  BusinessDetailParams _businessDetails;
  bool _isEmailVisible = true;
  String _switchLabel = TextOf.USE_MOBILE_NUMBER;
  AutovalidateMode _autoValidateMode;

  _BusinessSignUpFormState(Data predefinedData, User user) {
    if (predefinedData != null) {
      _businessDetails = _populateBusinessDetails(predefinedData);
      _emailController.text = _businessDetails.emailAddress;
      _mobileController.text = _businessDetails.mobileNumber;
    } else {
      _businessDetails = BusinessDetailParams();
    }
    /*if (user == null) {
      if (predefinedData != null) {
        _businessDetails = _populateBusinessDetails(predefinedData);
        _emailController.text = _businessDetails.emailAddress;
        _mobileController.text = _businessDetails.mobileNumber;
      } else {
        _businessDetails = BusinessDetailParams();
      }
    } else {
      _businessDetails = _populateBusinessDetails(predefinedData);
      _businessDetails.emailAddress = user.responseData.emailAddress;
      _businessDetails.mobileNumber = user.responseData.mobileNumber;
      _emailController.text = _businessDetails.emailAddress;
      _mobileController.text = _businessDetails.mobileNumber;
      _businessDetails.userId = user.responseData.id;
      if (_businessDetails.emailAddress.isEmpty) {
        _isEmailVisible = false;
        _switchLabel = TextOf.USE_EMAIL_ADDRESS;
      }
    }*/
  }

  @override
  void initState() {
    _autoValidateMode = AutovalidateMode.disabled;
    if (widget.loginFromEmail != null && !widget.loginFromEmail) {
      _isEmailVisible = false;
      _switchLabel = TextOf.USE_EMAIL_ADDRESS;
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    if (widget.apiKey == null || widget.apiKey.isEmpty)
      throw Exception(General.API_KEY_REQUIRED);
    return WillPopScope(
        child: Scaffold(
            key: _scaffoldStateKey,
            appBar: AppBar(
                title: Text(Titles.WELCOME),
                leading: logo,
                automaticallyImplyLeading: Platform.isIOS
            ),
            body: Padding(
              padding: PaddingDetails.FORM,
              child: Form(
                key: _formKey,
                autovalidateMode: _autoValidateMode,
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Expanded(
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          // Welcome note
                          Padding(
                            padding: PaddingDetails.NOTE,
                            child: Text(
                              TextOf.NOTE,
                              style: TextStyle(
                                  color: Colors.grey,
                                  fontSize: Layout.dp18
                              ),
                            ),
                          ),
                          // Email or Mobile
                          Padding(
                            padding: PaddingDetails.FIELD_TOP,
                            child: Stack(
                              children: [
                                // Email
                                Visibility(
                                  visible: _isEmailVisible,
                                  child: TextFormField(
                                    controller: _emailController,
                                    decoration: InputDecoration(
                                        border: const OutlineInputBorder(),
                                        labelText: FormLabels.EMAIL_ADDRESS
                                    ),
                                    validator: (value) {
                                      if (value.isEmpty || !value.contains(RegExp(Texts.REGEX)))
                                        return Utils.showEnterFieldError("email address");
                                      else
                                        return null;
                                    },
                                    keyboardType: TextInputType.emailAddress,
                                    inputFormatters: [
                                      LengthLimitingTextInputFormatter(200),
                                    ],
                                  ),
                                ),
                                // Mobile
                                Visibility(
                                  visible: !_isEmailVisible,
                                  child: TextFormField(
                                    controller: _mobileController,
                                    decoration: InputDecoration(
                                        border: const OutlineInputBorder(),
                                        labelText: FormLabels.MOBILE_NUMBER
                                    ),
                                    validator: (value) {
                                      if (value.isEmpty || value.length < 11)
                                        return Utils.showEnterFieldError("mobile number");
                                      else
                                        return null;
                                    },
                                    keyboardType: TextInputType.number,
                                    inputFormatters: [
                                      LengthLimitingTextInputFormatter(11)
                                    ],
                                  ),
                                )
                              ],
                            ),
                          ),
                          // Switch text
                          Padding(
                            padding: PaddingDetails.HINT,
                            child: InkWell(
                              child: Text(
                                _switchLabel,
                                style: TextStyle(
                                    fontWeight: FontWeight.bold,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                              onTap: _switchField,
                            ),
                          ),
                        ],
                      ),
                    ),
                    // Next button
                    Container(
                      width: double.infinity,
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.end,
                        children: [
                          Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              Utils.getHelplineMark(),
                              Utils.getLogoMark(),
                            ],
                          ),
                          SizedBox(
                            width: Layout.BTN_WIDTH,
                            child: ElevatedButton(
                              key: Key("btnNext"),
                              onPressed: () {
                                if (_formKey.currentState.validate()) {
                                  FocusScope.of(context).unfocus();
                                  if (widget.data != null && widget.data.isNotEmpty) {
                                    _businessDetails.emailAddress = _emailController.value.text;
                                    _businessDetails.mobileNumber = _mobileController.value.text;
                                    if (_isEmailVisible) {
                                      acquireUser(AcquireUser(
                                          emailAddress: _emailController.value.text,
                                          mobileNumber: "",
                                          isVerificationRequired: false
                                      ));
                                    } else {
                                      acquireUser(AcquireUser(
                                          emailAddress: "",
                                          mobileNumber: _mobileController.value.text,
                                          isVerificationRequired: false
                                      ));
                                    }
                                  }
                                } else {
                                  setState(() {
                                    _autoValidateMode = AutovalidateMode.always;
                                  });
                                }
                              },
                              child: Text(ButtonTags.NEXT),
                            ),
                          )
                        ],
                      ),
                    )
                  ],
                ),
              ),
            )
        ),
        onWillPop: () {
          Navigator.of(context).pop(false);
          return Future.value(false);
        }
    );
  }
  BusinessDetailParams _populateBusinessDetails(Data predefinedData) {
    return BusinessDetailParams.fromJson(predefinedData.toJson());
  }
  void _switchField() {
    FocusScope.of(context).unfocus();
    setState(() {
      if (_isEmailVisible) {
        _isEmailVisible = false;
        _switchLabel = TextOf.USE_EMAIL_ADDRESS;
      } else {
        _isEmailVisible = true;
        _switchLabel = TextOf.USE_MOBILE_NUMBER;
      }
    });
  }
  void verifyUserFlow(User user) {
    Navigator.pushReplacement(context, MaterialPageRoute(builder: (context) =>
        VerifyUser(
          apiKey: widget.apiKey,
          data: widget.data,
          requiredEmailOTP: _isEmailVisible,
          preDefinedData: widget.predefinedData,
          user: user,
          businessDetails: _businessDetails,
          callbackBusinessId: widget.callbackBusinessId,
        )
    )).then((value) {
      if (value != null && value[0] && value[1])
        Navigator.of(context).pop(value[0] && value[1]);
    });
  }
  void loginOrCreatePasswordFlow({bool isLogin, User user}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) =>
        PasswordScreen(
          apiKey: widget.apiKey,
          data: widget.data,
          isLogin: isLogin,
          user: user,
          businessDetails: _businessDetails,
          callbackBusinessId: widget.callbackBusinessId,
        )
    )).then((value) {
      if (value != null && value[0] && value[1])
        Navigator.of(context).pop(value[0] && value[1]);
    });
  }
  void acquireUser(AcquireUser acquireUser) {
    Utils.showLoaderDialogWithText(context, "Please wait...");
    Utils.acquireUser(widget.apiKey, acquireUser).then((user) {
      if (user != null) {
        if (user.responseCode == 0) {
          if (isEmailOrMobileNumber(user)) {
            Navigator.of(context).pop(false);
            loginOrCreatePasswordFlow(
                isLogin: user.responseData.password.isNotEmpty,
                user: user
            );
          } else {
            Utils.requestOtp(widget.apiKey, OtpParams(
                userId: user.responseData.id
            ), _isEmailVisible
            ).then((response) {
              if (response != null) {
                if (response.requiresCode == 0) {
                  Navigator.of(context).pop(false);
                  verifyUserFlow(user);
                } else {
                  Navigator.of(context).pop(false);
                  Utils.showSnackBar(_scaffoldStateKey
                      .currentState, response
                      .responseData.errorMessage
                  );
                }
              }
            }).timeout(
                Duration(seconds: Utils.timeOutSec),
                onTimeout: () {
                  Navigator.of(context).pop(false);
                  Utils.showSnackBar(_scaffoldStateKey.currentState,
                      GeneralError.SERVICE_UNAVAILABLE);
                }
            ).catchError((onError) {
              print(onError);
              Navigator.of(context).pop(false);
              Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
            });
          }
        }
      }
    }).timeout(
        Duration(seconds: Utils.timeOutSec),
        onTimeout: () {
          Navigator.of(context).pop(false);
          Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
        }
    ).catchError((onError) {
      print(onError);
      Navigator.of(context).pop(false);
      Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
    });
  }
  bool isEmailOrMobileNumber(User user) {
    return (_isEmailVisible && user.responseData.isEmailVerified) ||
        (!_isEmailVisible && user.responseData.isMobileNumberVerified);
  }
}