library businessonboarding;

import 'dart:async';
import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:package_info/package_info.dart';

import 'src/Constants.dart';
import 'src/Responses/AppVersionInfoResponse.dart';
import 'src/Utilities/Utils.dart';

class OnBoardingForm {
  Future<Map<String, String>> build({@required BuildContext context,
    @required String apiKey, bool isTesting }) async {

    throwErrorIfDataIsMissing(context, apiKey);
    if (isTesting != null) Utils.isTesting = isTesting;
    Map<String, String> mappedData = new Map();
    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    if (packageInfo != null) {
      Utils.showLoaderDialogWithText(context, "Loading...");
      try {
        AppVersionInfoResponse infoResponse = await Utils.versionInfoCall(apiKey, packageInfo.version);
        if (infoResponse.responseCode == 200) {
          if (infoResponse.isUpdateAvailable)
            print(GeneralWarning.VERSION_UPDATE + infoResponse.availableSDKVersion);

          await Utils.fetchLegalEntities(apiKey).then((value) => mappedData[General.MAP_KEY_LEGAL_ENTITIES] = value);
          await Utils.fetchBusinessCategories(apiKey).then((value) => mappedData[General.MAP_KEY_BUSINESS_CATEGORIES] = value);
          await Utils.fetchBanks(apiKey).then((value) => Utils.banks = value);
          await Utils.fetchCities(apiKey).then((value) => Utils.cities = value);
          await Utils.fetchFeeBorne(apiKey).then((value) => Utils.feeBorne = value);
          await Utils.fetchProvinces(apiKey).then((value) => Utils.provinces = value);
          await Utils.fetchInstrumentTypes(apiKey).then((value) => Utils.instrumentType = value);

          Navigator.of(context).pop(false);
          return mappedData;
        } else {
          Navigator.of(context).pop(false);
          throw Exception(infoResponse.responseCode);
        }

      } on DioError catch(response) {
        Navigator.of(context).pop(false);
        if (response.type == DioErrorType.RESPONSE) {
          throw(response.response.data);
        } else if (response.type == DioErrorType.DEFAULT) {
          throw(json.encode({
            "response_code": 000,
            "response_message": "Please check your internet connection"
          }));
        }
      } on TimeoutException catch(_) {
        Navigator.of(context).pop(false);
        throw(json.encode({
          "response_code": 000,
          "response_message": "Request Timeout"
        }));
      } catch(error) {
        Navigator.of(context).pop(false);
        throw(error);
      }
    } else {
      Navigator.of(context).pop(false);
      throw(json.encode({
        "response_code": 000,
        "response_message": "Cannot read package info"
      }));
    }
    Navigator.of(context).pop(false);
    return null;
  }

  void throwErrorIfDataIsMissing(BuildContext context, String key) {
    if (context == null) {
      throw(json.encode({
        "response_code": -1,
        "response_message": "Please provide build context"
      }));
    }
    if (key == null || key.isEmpty) {
      throw(json.encode({
        "response_code": -1,
        "response_message": "Api key cannot be null or empty"
      }));
    }
  }
}