import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import 'Constants.dart';
import 'PaymentMethod.dart';
import 'Requests/BusinessDetailParams.dart';
import 'Requests/PartnerDetailsParams.dart';
import 'Utilities/Utils.dart';

class AdditionalBusinessDetails extends StatefulWidget {
  final String apiKey;
  final logo;
  final BusinessDetailParams businessDetails;
  final ValueSetter<String> callbackBusinessId;
  AdditionalBusinessDetails({Key key, @required this.apiKey,
    this.logo, this.businessDetails, this.callbackBusinessId}) : super(key: key);
  @override
  AdditionalBusinessDetailsState createState() =>
      AdditionalBusinessDetailsState();
}
class AdditionalBusinessDetailsState extends State<AdditionalBusinessDetails> {
  final _formKey = GlobalKey<FormState>();
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();

  Map<String, String> banks = Map();
  bool _showPartnersDetails;
  bool _showNtn;
  bool _multipleFieldsRequired;
  AutovalidateMode _autoValidateMode;
  var urlPattern = r"([-A-Z0-9.]+)(/[-A-Z0-9+&@#/%=~_|!:,.;]*)?(\?[A-Z0-9+&@#/%=~_|!:‌​,.;]*)?";
  int index;
  static List<Widget> partnersList = List();
  static Map<int, String> partnerName = Map();
  static Map<int, String> partnerCnic = Map();
  List<dynamic> _partnersDetails;
  List<int> matchedPartnerDetailsIndexes = List();

  @override
  void initState() {
    _autoValidateMode = AutovalidateMode.disabled;
    partnersList = List();
    index = 2;
    if (widget.businessDetails.businessType.id == 3
        || widget.businessDetails.businessType.id == 4 ||
        widget.businessDetails.businessType.id == 5)
    {
      _showPartnersDetails = true;
      _showNtn = true;
      _multipleFieldsRequired = true;
    } else {
      _showPartnersDetails = false;
      _showNtn = false;
      _multipleFieldsRequired = false;
    }
    _partnersDetails = List();
    _populatePartnerDetails(widget.businessDetails);
    super.initState();
  }

  void removeField(int index){
    FocusScope.of(context).requestFocus(new FocusNode());
    setState(() {
      this.index--;
      partnerName.removeWhere((key, value) => key == index);
      partnerCnic.removeWhere((key, value) => key == index);
      partnersList.removeWhere((element) => element.key == Key("index_$index"));
    });
  }

  String comparePartners(){
    if(partnerCnic[2].length == 13 && partnerCnic[2] == widget.businessDetails.cnicOfBusinessOwner) return partnerCnic[2];
    else if(matchedPartnerDetailsIndexes.isNotEmpty)
      if(partnerCnic[matchedPartnerDetailsIndexes[0]] == partnerCnic[matchedPartnerDetailsIndexes[1]] || widget.businessDetails.cnicOfBusinessOwner == partnerCnic[matchedPartnerDetailsIndexes[0]]) return partnerCnic[matchedPartnerDetailsIndexes[0]];
    else return "no same partner";
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        child: Scaffold(
            key: _scaffoldStateKey,
            appBar: AppBar(
              title: Text(Titles.ADDITIONAL_BUSINESS_DETAILS),
              leading: widget.logo,
              automaticallyImplyLeading: Platform.isIOS,
            ),
            body: Padding(
              padding: PaddingDetails.FORM,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Expanded(
                    child: SingleChildScrollView(
                      child: Form(
                        key: _formKey,
                        autovalidateMode: _autoValidateMode,
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            // Subtitle
                            Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: Text(
                                SubTitle.ADDITIONAL_BUSINESS_DETAILS_DESCRIPTION,
                                style: TextStyling.getHeaderStyle(),
                              ),
                            ),
                            // Name of business owner
                            Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: TextFormField(
                                // controller: _nameOfBusinessOwnerController,
                                decoration: InputDecoration(
                                    border: const OutlineInputBorder(),
                                    labelText: FormLabels.NAME_OF_BUSINESS_OWNER
                                ),
                                validator: (value) {
                                  if (value.isEmpty) return Utils.showEnterFieldError("Name of business owner");
                                  else if (value.length < LengthLimit.MIN_LENGTH_5) return ErrorFields.MIN_LENGTH_ERROR_5;
                                  else if (value.length > LengthLimit.MAX_LENGTH_100) return ErrorFields.MAX_LENGTH_ERROR_100;
                                  else return null;
                                },
                                initialValue: widget.businessDetails.nameOfBusinessOwner,
                                onChanged: (value) => widget.businessDetails.nameOfBusinessOwner = value,
                              ),
                            ),
                            // Hint
                            Padding(
                              padding: PaddingDetails.HintPadding,
                              child: Text(HelpText.BUSINESS_OWNER),
                            ),
                            // CNIC# of business owner
                            Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: TextFormField(
                                // controller: _cnicOfBusinessOwnerController,
                                decoration: InputDecoration(
                                    border: const OutlineInputBorder(),
                                    labelText: FormLabels.CNIC_OF_BUSINESS_OWNER
                                ),
                                validator: (value) {
                                  if(value.isEmpty) return "CNIC is required";
                                  else if (value.length < 13 ) return "CNIC must contain 13 digits";
                                  else if (_showPartnersDetails) if (value == comparePartners()) return "CNIC's are Same";
                                  else return null;
                                },
                                inputFormatters: [
                                  LengthLimitingTextInputFormatter(13),
                                  FilteringTextInputFormatter.allow(RegExp("[0-9]"))
                                ],
                                keyboardType: TextInputType.phone,
                                initialValue: widget.businessDetails.cnicOfBusinessOwner,
                                onChanged: (value) => widget.businessDetails.cnicOfBusinessOwner = value,
                              ),
                            ),
                            // Partner/Director information
                            Visibility(
                              visible: _showPartnersDetails,
                              child: Column(
                                children: [
                                  // Partner/Director 2 info
                                  Padding(
                                      padding: PaddingDetails.FIELD_TOP,
                                      child: TextFormField(
                                        decoration: InputDecoration(
                                            border: const OutlineInputBorder(),
                                            labelText: FormLabels.DIRECTOR_2_NAME
                                        ),
                                        validator: (val) {
                                          if (val.isEmpty)
                                            return Utils.showEnterFieldError("partner name");
                                          else if (val.length < LengthLimit.MIN_LENGTH_5)
                                            return ErrorFields.MIN_LENGTH_ERROR_5;
                                          else if (val.length > LengthLimit.MAX_LENGTH_100)
                                            return ErrorFields.MAX_LENGTH_ERROR_100;
                                          else return null;
                                        },
                                        onChanged: (val) => partnerName[2] = val.trim(),
                                        textCapitalization: TextCapitalization.sentences,
                                        initialValue: partnerName[2] ?? "",
                                      )
                                  ),
                                  Padding(
                                      padding: PaddingDetails.FIELD_TOP,
                                      child: TextFormField(
                                        decoration: InputDecoration(
                                            border: const OutlineInputBorder(),
                                            labelText: FormLabels.DIRECTOR_2_CNIC
                                        ),
                                        validator: (value) {
                                          if(value.isEmpty) return "CNIC is required";
                                          else if (value.length < 13 ) return "CNIC must contain 13 digits";
                                          else if (value == comparePartners()) return "CNIC's are Same";
                                          return null;
                                        },
                                        keyboardType: TextInputType.phone,
                                        inputFormatters: [
                                          LengthLimitingTextInputFormatter(13),
                                          FilteringTextInputFormatter.allow(RegExp("[0-9]"))
                                        ],
                                        onChanged: (val) => partnerCnic[2] = val,
                                        initialValue: partnerCnic[2] ?? "",
                                      )
                                  ),
                                  Visibility(
                                      visible: _multipleFieldsRequired,
                                      child: ListBody(
                                          children: partnersList // _getPartnersDetailsFields(),
                                      )
                                  ),
                                  // Text to add partners
                                  Align(
                                    alignment: Alignment.centerRight,
                                    child: Padding(
                                      padding: PaddingDetails.MORE_PARTNERS,
                                      child: InkWell(
                                        onTap: () {
                                          setState(() {
                                            _multipleFieldsRequired = true;
                                            partnersList.add(_getFields(++index));
                                          });
                                        },
                                        child: Text(
                                            TextOf.ADD_MORE,
                                            style: TextStyle(
                                                fontWeight: FontWeight.bold,
                                                color: Colors.blue,
                                                decoration: TextDecoration.underline
                                            )
                                        ),
                                      ),
                                    ),
                                  ),
                                ],
                              ),
                            ),
                            // NTN
                            Visibility(
                              visible: _showNtn,
                              child: Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: [
                                    TextFormField(
                                      // controller: _ntnController,
                                      decoration: InputDecoration(
                                          border: const OutlineInputBorder(),
                                          labelText: FormLabels.NTN
                                      ),
                                      validator: (value) {
                                        if (value == null) {
                                          return Utils.showEnterFieldError("NTN");
                                        } else if (value.length < LengthLimit.MIN_LENGTH_3) {
                                          return ErrorFields.MIN_LENGTH_ERROR_3;
                                        } else if (value.length > LengthLimit.MAX_LENGTH_14) {
                                          return ErrorFields.MAX_LENGTH_ERROR_14;
                                        } else {
                                          return null;
                                        }
                                      },
                                      keyboardType: TextInputType.number,
                                      inputFormatters: [
                                        LengthLimitingTextInputFormatter(14),
                                        FilteringTextInputFormatter.allow(RegExp("[0-9]"))
                                      ],
                                      initialValue: widget.businessDetails.ntn,
                                      onChanged: (value) => widget.businessDetails.ntn = value,
                                    ),
                                    Padding(
                                      padding: PaddingDetails.HintPadding,
                                      child: Text(HelpText.NTN),
                                    ),
                                  ],
                                ),
                              ),
                            ),
                            // Official website
                            Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: TextFormField(
                                // controller: _officialWebsiteController,
                                decoration: InputDecoration(
                                  border: const OutlineInputBorder(),
                                  labelText: FormLabels.OFFICIAL_WEBSITE,
                                ),
                                validator: (value) {
                                  if (value.isNotEmpty && !RegExp(urlPattern).hasMatch(value))
                                    return "Please enter a valid website";
                                  return null;
                                },
                                initialValue: widget.businessDetails.officialWebsite,
                                onChanged: (value) => widget.businessDetails.officialWebsite = value,
                              ),
                            ),
                            // Expected Monthly TurnOver
                            Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: TextFormField(
                                  // controller: _expectedTurnOverController,
                                  decoration: InputDecoration(
                                      border: const OutlineInputBorder(),
                                      labelText: FormLabels.EXPECTED_MONTHLY_TURNOVER,
                                      prefix: Text(General.PAK_RUPEE)
                                  ),
                                  validator: (value) {
                                    if (value.isEmpty)
                                      return Utils.showEnterFieldError("expected monthly turnover");
                                    else return null;
                                  },
                                  keyboardType: TextInputType.phone,
                                  inputFormatters: [
                                    LengthLimitingTextInputFormatter(12),
                                    FilteringTextInputFormatter.allow(RegExp("[0-9]"))
                                  ],
                                  initialValue: widget.businessDetails.expectedMonthlyTurnOver,
                                  onChanged: (value) => widget.businessDetails.expectedMonthlyTurnOver = value,
                                )
                            ),
                          ],
                        ),
                      ),
                    ),
                  ),
                  Padding(
                    padding: PaddingDetails.NOTE,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.stretch,
                      children: [
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            Utils.getHelplineMark(),
                            Utils.getLogoMark(),
                          ],
                        ),
                        Row(
                          children: [
                            // Button Save
                            Expanded(
                              flex: 2,
                              child: Padding(
                                padding: PaddingDetails.BTN_RIGHT,
                                child: SizedBox(
                                  width: Layout.BTN_WIDTH,
                                  child: ElevatedButton(
                                    onPressed: () async {
                                      if(setPartnerDetailsIfAny()){
                                        onBoardBusiness(
                                            isSubmitting: false,
                                            moveToNext: false,
                                        );
                                        FocusScope.of(context).requestFocus(new FocusNode());
                                      }
                                    },
                                    child: Text(
                                      ButtonTags.SAVE,
                                      style: TextStyle(
                                        color: Theme.of(context).primaryColor,
                                      ),
                                    ),
                                    style: ButtonStyle(
                                        backgroundColor: MaterialStateColor
                                            .resolveWith((states) => Colors.white)
                                    ),
                                  ),
                                ),
                              ),
                            ),
                            // Button Next
                            Expanded(
                              flex: 2,
                              child: Padding(
                                padding: PaddingDetails.BTN_LEFT,
                                child: SizedBox(
                                  width: Layout.BTN_WIDTH,
                                  child: ElevatedButton(
                                      onPressed: () async {
                                        if (setPartnerDetailsIfAny()) {
                                          if(_formKey.currentState.validate()){
                                            onBoardBusiness(
                                              isSubmitting: false,
                                              moveToNext: true,
                                            );
                                            FocusScope.of(context).requestFocus(new FocusNode());
                                          }
                                        }
                                        else {
                                          setState(() {
                                            _autoValidateMode = AutovalidateMode.always;
                                          });
                                        }
                                      },
                                      child: Text(ButtonTags.NEXT)
                                  ),
                                ),
                              ),
                            )
                          ],
                        )
                      ],
                    ),
                  )
                ],
              ),
            )
        ),
        onWillPop: () {
          Navigator.of(context).pop(false);
          return Future.value(false);
        }
    );
  }
  void onBoardBusiness({bool isSubmitting, bool moveToNext}) async {
    widget.businessDetails.isSubmitting = isSubmitting;
    print(widget.businessDetails.toJson());
    Utils.apiCallsHandler(context, _scaffoldStateKey.currentState, () async {
      moveToNext ? Utils.showLoaderDialogWithText(context, "Please wait...")
          : Utils.showLoaderDialogWithText(context, "Saving...");
      await Utils.onBoardBusiness(widget.apiKey, widget.businessDetails).then((response) {
        if (response != null) {
          var data = json.decode(response);
          var responseCode = data['response_code'];
          var responseMessage = data['response_message'];
          if (responseCode == 0) {
            Navigator.of(context).pop(false);
            if (!moveToNext)
              widget.callbackBusinessId(widget.businessDetails.businessId);
            moveToNext ? paymentMethodFlow(widget.businessDetails) : Utils.showSnackBar(
                _scaffoldStateKey.currentState, General.showSaveResponse2());
          } else {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, responseMessage);
          }
        }
      }).timeout(
          Duration(seconds: Utils.timeOutSec),
          onTimeout: () {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
          }
      ).catchError((onError) {
        print(onError);
        Navigator.of(context).pop(false);
        Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
      });
    });
  }
  void paymentMethodFlow(BusinessDetailParams businessDetail) {
    Navigator.push(context, MaterialPageRoute(
        builder: (context) => PaymentMethod(
          apiKey: widget.apiKey,
          businessDetails: businessDetail,
          callbackBusinessId: widget.callbackBusinessId,
        )
    )).then((value) {
      if (value != null && value[0] && value[1])
        Navigator.of(context).pop(value);
    });
  }
  void _populatePartnerDetails(BusinessDetailParams params) {
    if (params.partnerDetails != null && params.partnerDetails.isNotEmpty) {
      bool isFirstIterationComplete = false;
      int count = 2;
      List<PartnerDetailsParams> partners = List();
      for (var partner in params.partnerDetails)
        partners.add(PartnerDetailsParams.fromJson(partner));
      for (PartnerDetailsParams details in partners) {
        partnerName[count] = details.name;
        partnerCnic[count] = details.cnic;
        count++;
        if (isFirstIterationComplete) {
          index++;
          partnersList.add(_getFields(index));
        }
        isFirstIterationComplete = true;
      }
    }
  }

  Widget _getFields(int index) {
    print(index);
    return Stack(
      key: Key("index_$index"),
      alignment: Alignment.centerRight,
      children: [
        PartnerDetailsFields(
          key: Key("index_$index"),
          index: index,
          notifyParent: removeField,
          matchedPartners: comparePartners,
        ),
      ],
    );
  }

  bool setPartnerDetailsIfAny() {
    matchedPartnerDetailsIndexes.clear();
    bool validate = true;
    if (_showPartnersDetails) {
      if(widget.businessDetails.cnicOfBusinessOwner == partnerCnic[2]){
        Utils.showSnackBar(_scaffoldStateKey.currentState, "Owner and Partner CNIC's Are Same");
        return false;
      } // if there is only one partner
      if (_multipleFieldsRequired) {
        _partnersDetails.clear();
        for (int i = 2; i <= index; ++i) {
          var te = PartnerDetailsParams(
              name: partnerName[i],
              cnic: partnerCnic[i]
          ).toJson();
          for(int j = 2; j <= partnerCnic.length; j++){
              if(j != i){
                if(widget.businessDetails.cnicOfBusinessOwner == partnerCnic[j] || widget.businessDetails.cnicOfBusinessOwner == partnerCnic[i] || partnerCnic[i] == partnerCnic[j]){
                    validate = false;
                    matchedPartnerDetailsIndexes.add(i);
                    matchedPartnerDetailsIndexes.add(j);
                    print(matchedPartnerDetailsIndexes.toString());
                    Utils.showSnackBar(_scaffoldStateKey.currentState, "Partner CNIC's Are Same");
                    return false;
                }
              }
          }
          if(validate) _partnersDetails.add(te);
        }
      }
      widget.businessDetails.partnerDetails = _partnersDetails;
      return validate;
    }
    return validate;
  }
}

class PartnerDetailsFields extends StatefulWidget {
  final int index;
  final Function notifyParent;
  final Function matchedPartners;
  PartnerDetailsFields({Key key, this.index, this.notifyParent, this.matchedPartners}) : super(key: key);
  @override
  _PartnerDetailsFieldsState createState() => _PartnerDetailsFieldsState();
}
class _PartnerDetailsFieldsState extends State<PartnerDetailsFields> {
  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        // Partner/Director Name
        Padding(
          padding: PaddingDetails.FIELD_TOP,
          child: TextFormField(
            decoration: InputDecoration(
              border: OutlineInputBorder(),
              labelText: Utils.showPartnerDirectorNameText(widget.index),
              suffixIcon: IconButton(
                icon: Icon(Icons.delete),
                color: Colors.red,
                onPressed: (){
                  widget.notifyParent(widget.index);
                },
              )
            ),
            validator: (val) {
              if (val.isEmpty) return "Please enter partner name";
              else return null;
            },
            textCapitalization: TextCapitalization.sentences,
            onChanged: (value) => AdditionalBusinessDetailsState
                .partnerName[widget.index] = value,
            initialValue: AdditionalBusinessDetailsState
                .partnerName[widget.index],
          ),
        ),
        // Partner/Director CNIC
        Padding(
            padding: PaddingDetails.FIELD_TOP,
            child: TextFormField(
              decoration: InputDecoration(
                  border: const OutlineInputBorder(),
                  labelText: Utils.showPartnerDirectorCnicText(widget.index)
              ),
              validator: (value) {
                if(value.isEmpty) return "CNIC is required";
                else if (value.length < 13 ) return "CNIC must contain 13 digits";
                else if (value == widget.matchedPartners()) return "CNIC's are Same";
                else return null;
              },
              keyboardType: TextInputType.phone,
              inputFormatters: [
                LengthLimitingTextInputFormatter(13),
                FilteringTextInputFormatter.allow(RegExp("[0-9]"))
              ],
              onChanged: (value) => AdditionalBusinessDetailsState
                  .partnerCnic[widget.index] = value,
              initialValue: AdditionalBusinessDetailsState
                  .partnerCnic[widget.index],
            )
        ),
      ],
    );
  }
  @override
  void dispose() {
    super.dispose();
  }
}