import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import 'AdditionalBusinessDetails.dart';
import 'Constants.dart';
import 'Requests/BusinessDetailParams.dart';
import 'Responses/User.dart';
import 'RetrofitConfigForee.dart';
import 'Utilities/Utils.dart';

class BusinessDetails extends StatefulWidget {
  final String apiKey;
  final User user;
  final BusinessDetailParams businessDetails;
  final Map<String, String> data;
  final ValueSetter<String> callbackBusinessId;
  BusinessDetails({Key key, @required this.apiKey, this.data,
    this.user, this.businessDetails, this.callbackBusinessId}) : super(key: key);
  @override
  _BusinessDetailsState createState() => _BusinessDetailsState(apiKey, data);
}
class _BusinessDetailsState extends State<BusinessDetails> {
  final _formKey = GlobalKey<FormState>();
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();

  Widget logo;
  int _idOfBusinessType;
  int _idOfBusinessCategory;
  String otherTypeOfBusiness;
  String businessCategory;
  String bankName;
  String logoPath = "";

  bool showLogoUploadIcon = true;
  bool isLegalEntityOtherFieldVisible = false;
  bool isBusinessCategoryOtherFieldVisible = false;
  bool isSaveButtonDisabled = true;
  bool isLogoUploaded = false;

  Map<int, String> businessTypes;
  Map<int, String> businessCategoriesMap;
  Map<String, String> banksMap;
  Map<int, dynamic> partnerDetails;

  File file;
  BankAccountDetails businessDetails;
  AutovalidateMode _autoValidateMode;
  // MaskTextInputFormatter _maskFormatter;

/*  TextEditingController _businessIdController;
  TextEditingController _businessNameController;
  TextEditingController _businessDescriptionController;
  TextEditingController _emailController;
  TextEditingController _mobileController;*/

  _BusinessDetailsState(String apiKey, Map<String, String> data) {
    if (data != null) {
      if (data != null) {
        if (data.containsKey(General.MAP_KEY_LEGAL_ENTITIES)) {
          businessTypes = Map();
          final rawData = data[General.MAP_KEY_LEGAL_ENTITIES];
          if (rawData != null) {
            for (var json in json.decode(rawData)) {
              BusinessType entities = BusinessType.fromJson(json);
              businessTypes[entities.id] = entities.name;
            }
          }
        }
      }
      if (data.containsKey(General.MAP_KEY_BUSINESS_CATEGORIES)) {
        businessCategoriesMap = new Map();
        for (var json in json.decode(
            data[General.MAP_KEY_BUSINESS_CATEGORIES])) {
          BusinessCategories categories = BusinessCategories.fromJson(json);
          businessCategoriesMap[categories.id] = categories.name;
        }
      }
    }
  }

  @override
  void initState() {
    _autoValidateMode = AutovalidateMode.disabled;
    _idOfBusinessType = 1;
    _idOfBusinessCategory = 9;
    if (widget.businessDetails.businessType != null)
      _idOfBusinessType = widget.businessDetails.businessType.id ?? 1;
    if (widget.businessDetails.businessCategories != null)
      _idOfBusinessCategory = widget.businessDetails.businessCategories.id ?? 9;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    if (businessTypes == null || businessCategoriesMap == null)
      throw Exception("Try again, Data are missing.");
    return WillPopScope(
        child: Scaffold(
            key: _scaffoldStateKey,
            appBar: AppBar(
              title: Text(Titles.BUSINESS_DETAILS),
              leading: logo,
              automaticallyImplyLeading: Platform.isIOS,
            ),
            body: Padding(
              padding: PaddingDetails.FORM,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  // Fields
                  Expanded(
                    child: SingleChildScrollView(
                      child: Form(
                        key: _formKey,
                        autovalidateMode: _autoValidateMode,
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            // Subtitle
                            Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: Text(
                                SubTitle.BUSINESS_DETAILS_DESCRIPTION,
                                style: TextStyling.getHeaderStyle(),
                              ),
                            ),
                            // Brand/Shop name
                            Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: TextFormField(
                                // controller: _businessNameController,
                                decoration: InputDecoration(
                                    border: const OutlineInputBorder(),
                                    labelText: FormLabels.BRAND_SHOP_NAME
                                ),
                                validator: (value) {
                                  if (value.isEmpty) {
                                    return Utils.showEnterFieldError("brand/shop name");
                                  } else
                                  if (value.length < LengthLimit.MIN_LENGTH_5) {
                                    return ErrorFields.MIN_LENGTH_ERROR_5;
                                  } else
                                  if (value.length > LengthLimit.MAX_LENGTH_200) {
                                    return ErrorFields.MAX_LENGTH_ERROR_200;
                                  } else
                                    return null;
                                },
                                initialValue: widget.businessDetails.businessName,
                                onChanged: (value) => widget.businessDetails.businessName = value,
                                textCapitalization: TextCapitalization.sentences,
                              ),
                            ),
                            // Hint
                            Padding(
                              padding: PaddingDetails.HintPadding,
                              child: Text(HelpText.BRAND_SHOP),
                            ),
                            // Business description
                            Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: TextFormField(
                                // controller: _businessDescriptionController,
                                decoration: InputDecoration(
                                    border: const OutlineInputBorder(),
                                    labelText: FormLabels.BUSINESS_DESCRIPTION
                                ),
                                validator: (value) {
                                  if (value.isEmpty) {
                                    return Utils.showEnterFieldError("business description");
                                  } else if (value.length < LengthLimit.MIN_LENGTH_5) {
                                    return ErrorFields.MIN_LENGTH_ERROR_5;
                                  } else if (value.length > LengthLimit.MAX_LENGTH_750)
                                    return ErrorFields.MAX_LENGTH_ERROR_750;
                                  else return null;
                                },
                                textCapitalization: TextCapitalization.sentences,
                                keyboardType: TextInputType.multiline,
                                maxLines: null,
                                inputFormatters: [
                                  LengthLimitingTextInputFormatter(LengthLimit.MAX_LENGTH_750)
                                ],
                                initialValue: widget.businessDetails.businessDescription,
                                onChanged: (value) => widget.businessDetails.businessDescription = value,
                              ),
                            ),
                            // Email Address
                            Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: TextFormField(
                                // controller: _emailController,
                                enabled: !widget.user.responseData.isEmailVerified,
                                decoration: InputDecoration(
                                  border: const OutlineInputBorder(),
                                  labelText: FormLabels.EMAIL_ADDRESS,
                                  counterText: General.EMPTY,
                                ),
                                validator: (value) {
                                  if (value.isNotEmpty && value.length < LengthLimit.MIN_LENGTH_5) {
                                    return Utils.showEnterFieldError("email address with minimum length of 5");
                                  } else if (value.isNotEmpty && value.length > LengthLimit.MAX_LENGTH_100) {
                                    return Utils.showEnterFieldError("email address with maximum length of 100");
                                  } else return null;
                                },
                                keyboardType: TextInputType.emailAddress,
                                initialValue: widget.businessDetails.emailAddress,
                                onChanged: (value) => widget.businessDetails.emailAddress = value,
                              ),
                            ),
                            // Mobile Number
                            Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: TextFormField(
                                  // controller: _mobileController,
                                  enabled: !widget.user.responseData.isMobileNumberVerified,
                                  decoration: InputDecoration(
                                    border: const OutlineInputBorder(),
                                    labelText: FormLabels.MOBILE_NUMBER,
                                    counterText: General.EMPTY,
                                  ),
                                  validator: (value) {
                                    if (value.isNotEmpty && value.length < 11)
                                      return Utils.showEnterFieldError("mobile number");
                                    else
                                      return null;
                                  },
                                  keyboardType: TextInputType.phone,
                                  inputFormatters: [
                                    LengthLimitingTextInputFormatter(11)
                                  ],
                                  initialValue: widget.businessDetails.mobileNumber,
                                  onChanged: (value) => widget.businessDetails.mobileNumber = value,
                                )
                            ),
                            // Type of business
                            Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: Column(
                                    children: [
                                      Container(
                                          child: DropdownButtonFormField(
                                            isExpanded: true,
                                            value: _idOfBusinessType,
                                            items: businessTypes.map((key, value) {
                                              return MapEntry(key, DropdownMenuItem<dynamic>(
                                                value: key,
                                                child: Text(value),
                                              )
                                              );
                                            }).values.toList(),
                                            validator: (value) {
                                              if (value == null)
                                                return Utils.showEnterFieldError("Type of Business");
                                              else return null;
                                            },
                                            decoration: InputDecoration(
                                                border: OutlineInputBorder(),
                                                labelText: FormLabels.TYPE_OF_BUSINESS
                                            ),
                                            iconEnabledColor: CupertinoColors
                                                .darkBackgroundGray,
                                            onTap: () => FocusScope.of(context)
                                                .requestFocus(FocusNode()),
                                            onChanged: (value) {
                                              if (value == 7) {
                                                setState(() {
                                                  _idOfBusinessType = value;
                                                  isLegalEntityOtherFieldVisible = true;
                                                });
                                              } else {
                                                setState(() {
                                                  isLegalEntityOtherFieldVisible = false;
                                                  _idOfBusinessType = value;
                                                });
                                              }
                                            },
                                          )
                                      ),
                                      Visibility(
                                          visible: isLegalEntityOtherFieldVisible,
                                          child: Padding(
                                            padding: const EdgeInsets.only(
                                                top: Layout.textViewTop
                                            ),
                                            child: TextFormField(
                                              decoration: InputDecoration(
                                                  border: const OutlineInputBorder(),
                                                  labelText: General.OTHER
                                              ),
                                              validator: (val) {
                                                if (val.isEmpty || otherTypeOfBusiness == null || otherTypeOfBusiness.isEmpty)
                                                  return ErrorFields.MANDATORY;
                                                else return null;
                                              },
                                              onChanged: (val) => otherTypeOfBusiness = val.trim(),
                                              textCapitalization: TextCapitalization.sentences,
                                              initialValue: otherTypeOfBusiness,
                                            ),
                                          )
                                      )
                                    ]
                                )
                            ),
                            // Business Category
                            Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: Column(
                                    crossAxisAlignment: CrossAxisAlignment.start,
                                    children: [
                                      DropdownButtonFormField(
                                        isExpanded: true,
                                        value: _idOfBusinessCategory,
                                        items: businessCategoriesMap.map((key, value) {
                                          return MapEntry(key, DropdownMenuItem<dynamic>
                                            (
                                            value: key,
                                            child: Text(value),
                                          )
                                          );
                                        }).values.toList(),
                                        validator: (value) {
                                          if (value == null)
                                            return Utils.showEnterFieldError("business category");
                                          else return null;
                                        },
                                        decoration: InputDecoration(
                                            border: OutlineInputBorder(),
                                            labelText: FormLabels.BUSINESS_CATEGORY
                                        ),
                                        iconEnabledColor: CupertinoColors
                                            .darkBackgroundGray,
                                        onTap: () => FocusScope.of(context)
                                            .requestFocus(FocusNode()),
                                        onChanged: (value) {
                                          setState(() {
                                            _idOfBusinessCategory = value;
                                          });
                                        },
                                      ),
                                      Padding(
                                        padding: PaddingDetails.HintPadding,
                                        child: Text(HelpText.BUSINESS_CATEGORY),
                                      ),
                                    ]
                                )
                            ),
                          ],
                        ),
                      ),
                    ),
                  ),
                  // Buttons
                  Padding(
                    padding: PaddingDetails.NOTE,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: [
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            Utils.getHelplineMark(),
                            Utils.getLogoMark(),
                          ],
                        ),
                        Row(
                          children: [
                            // Button save
                            Expanded(
                              flex: 2,
                              child: Padding(
                                padding: PaddingDetails.BTN_RIGHT,
                                child: SizedBox(
                                  width: Layout.BTN_WIDTH,
                                  child: ElevatedButton(
                                    onPressed: () {
                                      if (widget.businessDetails.businessName == null || widget.businessDetails.businessName.isEmpty) {
                                        Utils.showSnackBar(_scaffoldStateKey.currentState, "Please provide Brand/Shop Name first");
                                        return;
                                      }
                                      onBoardBusiness(
                                          isSubmitting: false,
                                          moveToNext: false
                                      );
                                      FocusScope.of(context).unfocus();
                                    },
                                    child: Text(
                                      ButtonTags.SAVE,
                                      style: TextStyle(
                                        color: Theme.of(context).primaryColor,
                                      ),
                                    ),
                                    style: ButtonStyle(
                                        backgroundColor: MaterialStateColor
                                            .resolveWith((states) => Colors.white)
                                    ),
                                  ),
                                ),
                              ),
                            ),
                            // Button Next
                            Expanded(
                              flex: 2,
                              child: Padding(
                                padding: PaddingDetails.BTN_LEFT,
                                child: SizedBox(
                                  width: Layout.BTN_WIDTH,
                                  child: ElevatedButton(
                                      onPressed: () async {
                                        if (_formKey.currentState.validate()) {
                                          onBoardBusiness(
                                              isSubmitting: false,
                                              moveToNext: true
                                          );
                                        } else {
                                          setState(() {
                                            _autoValidateMode = AutovalidateMode.always;
                                          });
                                        }
                                        // prepareData(false);
                                      },
                                      child: Text(ButtonTags.NEXT)
                                  ),
                                ),
                              ),
                            )
                          ],
                        )
                      ],
                    ),
                  )
                ],
              ),
            )
        ),
        onWillPop: () {
          Navigator.of(context).pop([true, false]);
          return Future.value(true);
        }
    );
  }
  BusinessCategories setBusinessCategory() {
    BusinessCategories category = BusinessCategories();
    category.id = _idOfBusinessCategory;
    category.name = businessCategoriesMap[_idOfBusinessCategory];
    return category;
  }
  BusinessType setBusinessType() {
    BusinessType type = BusinessType();
    type.id = _idOfBusinessType;
    type.name = businessTypes[_idOfBusinessType];
    return type;
  }
  void onBoardBusiness({bool isSubmitting, bool moveToNext}) async {
    widget.businessDetails.userId = widget.user.responseData.id;
    widget.businessDetails.isSubmitting = isSubmitting;
    widget.businessDetails.mobileNumber = widget.businessDetails.mobileNumber.replaceAll("-", "") ?? "";
    widget.businessDetails.businessType = setBusinessType();
    widget.businessDetails.businessCategories = setBusinessCategory();
    widget.businessDetails.supportedInstrument = widget.businessDetails.supportedInstrument?? List();
    print(widget.businessDetails.toJson());
    Utils.apiCallsHandler(context, _scaffoldStateKey.currentState, () async {
      moveToNext ? Utils.showLoaderDialogWithText(context, "Please wait...")
          : Utils.showLoaderDialogWithText(context, "Saving...");
      await Utils.onBoardBusiness(widget.apiKey, widget.businessDetails).then((response) {
        if (response != null) {
          var data = json.decode(response);
          var responseCode = data['response_code'];
          var responseMessage = data['response_message'];
          if (responseCode == 0) {
            var t = BusinessDetailParams.fromJson(data['response_data']['business']);
            widget.businessDetails.businessId = t.businessId;
            widget.businessDetails.settlementPeriod = t.settlementPeriod;
            Navigator.of(context).pop(false);
            if (!moveToNext)
              widget.callbackBusinessId(widget.businessDetails.businessId);
            moveToNext ? additionalBusinessDetailsFlow(widget.businessDetails)
                : Utils.showSnackBar(_scaffoldStateKey.currentState,
                General.showSaveResponse2());
          } else {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, responseMessage);
          }
        }
      }).timeout(
          Duration(seconds: Utils.timeOutSec),
          onTimeout: () {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
          }
      ).catchError((onError) {
        print(onError);
        Navigator.of(context).pop(false);
        Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
      });
    });
  }
  void additionalBusinessDetailsFlow(BusinessDetailParams businessDetails) {
    Navigator.push(_formKey.currentContext, MaterialPageRoute(
        builder: (context) =>
            AdditionalBusinessDetails(
              apiKey: widget.apiKey,
              businessDetails: businessDetails,
              callbackBusinessId: widget.callbackBusinessId,
            )
    )).then((value) {
      if (value != null && value[0] && value[1])
        Navigator.of(context).pop(value);
    });
  }
}