import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:http/http.dart' as http;

import 'package:dotted_border/dotted_border.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:image_picker/image_picker.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:url_launcher/url_launcher.dart';

import 'Requests/BusinessDetailParams.dart';
import 'Responses/ImageResponse.dart';
import 'RetrofitConfigForee.dart';
import 'Utilities/Utils.dart';
import 'Constants.dart';

class BusinessDocuments extends StatefulWidget {
  final String apiKey;
  final UserDetails userDetails;
  final Widget logo;
  final BusinessDetailParams businessDetails;
  final Map<String, dynamic> documents;
  final ValueSetter<String> callbackBusinessId;
  BusinessDocuments({Key key, this.apiKey, this.businessDetails, this.logo,
    this.userDetails, this.documents, this.callbackBusinessId}) : super(key: key);

  @override
  _BusinessDocumentsState createState() => _BusinessDocumentsState();
}
class _BusinessDocumentsState extends State<BusinessDocuments> {
  final _formKey = GlobalKey<FormState>();
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();
  int count;
  Map<String, bool> requiredDocs = {
    DocKeys.V2_DOC_BUSINESS_LOGO: false,
    DocKeys.V2_INCOME_PROOF: false,
    DocKeys.V2_DOC_CNIC_FRONT: false,
    DocKeys.V2_DOC_CNIC_BACK: false,
    DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE: false,
    DocKeys.V2_DOC_PARTNERSHIP_DEED: false,
    DocKeys.V2_DOC_BANK_ACC_MAINTENANCE_CERT: false,
    DocKeys.V2_DOC_MANDATE_LETTER: false,
    DocKeys.V2_DOC_MEMORANDUM_AND_ARTICLES: false,
    DocKeys.V2_DOC_INCORPORATION_CERTIFICATE: false,
    DocKeys.V2_DOC_SIGNING_AUTHORITY_BOARD_RESOLUTION: false,
    DocKeys.V2_DOC_COMMENCEMENT_CERTIFICATE: false,
    DocKeys.V2_DOC_DIRECTORS_LIST_UNDER_COMPANIES_ORDINANCE: false,
    DocKeys.V2_DOC_DIRECTORS_BANK_LETTER: false,
    DocKeys.V2_DOC_ALL_DIRECTORS_FROM_29: false,
    DocKeys.V2_DOC_BY_LAWS_AND_RULES: false,
    DocKeys.V2_DOC_RESOLUTION: false,
    DocKeys.V2_DOC_CONSENT_OR_AUTHORITY_LETTER: false,
    DocKeys.V2_DOC_REGISTARTION_CERTIFICATE: false,
    DocKeys.V2_DOC_ALL_TRUSTEES_BANK_LETTER: false,
    DocKeys.V2_DOC_LIST_OF_TRUSTEES_OR_MEMBERS: false,
    DocKeys.V2_DOC_UTILITY_BILL: false
  };

  Map<String, dynamic> documentsMap;
  bool _termsAndConditions = false;
  bool _isSubmitted = false;

  @override
  void initState() {
    documentsMap = Map();
    if (widget.documents != null && widget.documents.isNotEmpty) {
      documentsMap.addAll(widget.documents);
    }
    switch(widget.businessDetails.businessType.id) {
      case 2: {
        count = 3;
        break;
      }
      case 3: {
        count = 5;
        break;
      }
      case 4: {
        count = 8;
        break;
      }
      case 5: {
        count = 9;
        break;
      }
      case 6: {
        count = 9;
        break;
      }
      default: {
        count = 4;
      }
    }
    super.initState();
  }

  Widget drawDottedBox({Widget child}) {
    return DottedBorder(
      color: Colors.grey,
      strokeWidth: 1,
      dashPattern: [6, 3],
      strokeCap: StrokeCap.round,
      padding: const EdgeInsets.all(Layout.dp5),
      child: Container(
        width: double.infinity,
        height: Layout.uploadLogoH,
        child: child,
      ),
    );
  }

  bool showRequiredBox(int i1, int i2, int i3) {
    if (i2 != null && i3 != null) {
      return (widget.businessDetails.businessType.id == i1
          || widget.businessDetails.businessType.id == i2
          || widget.businessDetails.businessType.id == i3
      );
    } else {
      return widget.businessDetails.businessType.id == i1;
    }
  }

  Future<bool> checkFileType(String url) async {
    bool isImage;
    await http.get(url).then((value) {
      if(value.headers['content-type'].endsWith("pdf")) isImage = false;
      else isImage= true;
      });
    return Future.value(isImage);
  }

  Widget showPreviewOfUploadedFile(String document) {
    return(
        documentsMap.containsKey(document) ? FutureBuilder(
            initialData: documentsMap[document] ,
            future: checkFileType(documentsMap[document]),
            builder: (context, snapshot) {
              if(snapshot.hasData){
                if(snapshot.data == true) return Image.network(documentsMap[document]);
                if(snapshot.data == false) return Icon(Icons.picture_as_pdf, size: 42.0);
              }
              return CircularProgressIndicator();
            }
        ) : Utils.uploadIcon()
    );
  }

  bool checkCrossVisibility(String key) {
    return documentsMap == null ? false
        : (documentsMap.containsKey(key) && documentsMap[key].isNotEmpty);
  }

  Widget boxDescription({String title, String subTitle}) {
    return Expanded(
      child: SingleChildScrollView(
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Padding(
                padding: const EdgeInsets.only(top: 5.0, left: 5.0),
                child: Align(
                  alignment: Alignment.topLeft,
                  child: Text(
                    title,
                    softWrap: true,
                    style: TextStyle(
                        fontWeight: FontWeight.bold,
                        fontSize: 16.0
                    ),
                  ),
                ),
              ),
              Padding(
                padding: const EdgeInsets.only(top: 5.0, left: 5.0),
                child: Text(
                  subTitle,
                  softWrap: true,
                  style: TextStyle(
                      fontSize: 12.0
                  ),
                ),
              ),
            ],
          )
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        child: Scaffold(
            key: _scaffoldStateKey,
            appBar: AppBar(
              title: Text(Titles.BUSINESS_DOCUMENT),
              leading: widget.logo,
              automaticallyImplyLeading: Platform.isIOS,
            ),
            body: Padding(
              padding: PaddingDetails.FORM,
              child: Column(
                children: <Widget>[
                  // Fields
                  Expanded(
                      child: SingleChildScrollView(
                        child: Form(
                          key: _formKey,
                          child: Column(
                            children: [
                              /**
                               * Cnic
                               * Account maintenance certificate
                               */
                              Visibility(
                                visible: showRequiredBox(1, null , null),
                                child: Column(
                                  children: [
                                    // Cnic front,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_FRONT,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_FRONT)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_FRONT),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_FRONT)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_FRONT,
                                                      subTitle: DocumentDescription.CNIC_FRONT
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Cnic back,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_BACK,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_BACK)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_BACK),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_BACK)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_BACK,
                                                      subTitle: DocumentDescription.CNIC_BACK
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    //IncomeProof
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_INCOME_PROOF,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_INCOME_PROOF)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_INCOME_PROOF),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_INCOME_PROOF)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnPROOF_SOURCE_INCOME,
                                                      subTitle: DocumentDescription.PROOF_OF_SOURCE_OF_INCOME
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Account Maintenance certificate,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE)),
                                                          ),
                                                          /*// Logo of upload image.
                                                        Visibility(
                                                            visible: !checkCrossVisibility(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE),
                                                            child: Utils.uploadIcon()
                                                        )*/
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnM_CERTIFICATE,
                                                      subTitle: DocumentDescription.M_CERTIFICATE
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    )
                                  ],
                                ),
                              ),


                              Visibility(
                                visible: showRequiredBox(2, null, null),
                                child: Column(
                                  children: [
                                    // Cnic front,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_FRONT,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_FRONT)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_FRONT),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_FRONT)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_FRONT,
                                                      subTitle: DocumentDescription.CNIC_FRONT
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Cnic back,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_BACK,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_BACK)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_BACK),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_BACK)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_BACK,
                                                      subTitle: DocumentDescription.CNIC_BACK
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Account Maintenance certificate,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE)),
                                                          ),
                                                          /*// Logo of upload image.
                                                        Visibility(
                                                            visible: !checkCrossVisibility(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE),
                                                            child: Utils.uploadIcon()
                                                        )*/
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnM_CERTIFICATE,
                                                      subTitle: DocumentDescription.M_CERTIFICATE
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    )
                                  ],
                                ),
                              ),
                              /**
                               * Cnic of all partners,
                               * Partnership deed duly signed by all  partners of the firm,
                               * Account maintenance certificate by the name of all partners and length of relationship and
                               * Mandate letter
                               */
                              Visibility(
                                visible: showRequiredBox(3, null, null),
                                child: Column(
                                  children: [
                                    // All copies of partners
                                    // Cnic front,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_FRONT,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_FRONT)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_FRONT),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_FRONT)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_FRONT,
                                                      subTitle: DocumentDescription.CNIC_FRONT
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Cnic back,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_BACK,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_BACK)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_BACK),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_BACK)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_BACK,
                                                      subTitle: DocumentDescription.CNIC_BACK
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Partnership deed
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_PARTNERSHIP_DEED,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_PARTNERSHIP_DEED)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_PARTNERSHIP_DEED),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_PARTNERSHIP_DEED)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnPARTNERSHIP_DEED,
                                                      subTitle: DocumentDescription.PARTNERSHIP_DEED
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Account maintenance certificate
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_BANK_ACC_MAINTENANCE_CERT,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Container(
                                                height: Layout.uploadLogoH,
                                                child: Row(
                                                  children: [
                                                    Container(
                                                      width: Layout.uploadLogoW,
                                                      height: MediaQuery.of(context).size.height,
                                                      child: Card(
                                                        elevation: 0,
                                                        color: Colors.transparent,
                                                        child: Stack(
                                                          children: [
                                                            Center(
                                                                child: showPreviewOfUploadedFile(DocKeys.V2_DOC_BANK_ACC_MAINTENANCE_CERT)
                                                            ),
                                                            // Logo visibility of cross to remove image.
                                                            Visibility(
                                                              visible: checkCrossVisibility(DocKeys.V2_DOC_BANK_ACC_MAINTENANCE_CERT),
                                                              child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_BANK_ACC_MAINTENANCE_CERT)),
                                                            ),
                                                          ],
                                                        ),
                                                      ),
                                                    ),
                                                    boxDescription(
                                                        title: RequiredDocs.btnM_CERTIFICATE,
                                                        subTitle: DocumentDescription.PARTNER_M_CERTIFICATE
                                                    )
                                                  ],
                                                ),
                                              )
                                          ),
                                        )
                                    ),
                                    // Mandate letter
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_MANDATE_LETTER,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Container(
                                                height: Layout.uploadLogoH,
                                                child: Row(
                                                  children: [
                                                    Container(
                                                      width: Layout.uploadLogoW,
                                                      height: MediaQuery.of(context).size.height,
                                                      child: Card(
                                                        elevation: 0,
                                                        color: Colors.transparent,
                                                        child: Stack(
                                                          children: [
                                                            Center(
                                                                child: showPreviewOfUploadedFile(DocKeys.V2_DOC_MANDATE_LETTER)
                                                            ),
                                                            // Logo visibility of cross to remove image.
                                                            Visibility(
                                                              visible: checkCrossVisibility(DocKeys.V2_DOC_MANDATE_LETTER),
                                                              child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_MANDATE_LETTER)),
                                                            ),
                                                          ],
                                                        ),
                                                      ),
                                                    ),
                                                    boxDescription(
                                                        title: RequiredDocs.btnMANDATE_LETTER,
                                                        subTitle: DocumentDescription.MANDATE_LETTER
                                                    )
                                                  ],
                                                ),
                                              )
                                          ),
                                        )
                                    ),
                                  ],
                                ),
                              ),
                              /**
                               * Cnic of all directors,
                               * Public notary of memorandum & Articles of association,
                               * Public notary of certificate of incorporation
                               * Board resolution duly signed by company secretary,
                               * List of directors on form-A/Form-B issued under companies ordinance,
                               * Bank letter mentioning the name of all directors,
                               * List of all directors on form 29 issued by the registrar
                               */
                              Visibility(
                                visible: showRequiredBox(4, null, null),
                                child: Column(
                                  children: [
                                    // Cnic of all directors
                                    // Cnic front,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_FRONT,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_FRONT)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_FRONT),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_FRONT)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_FRONT,
                                                      subTitle: DocumentDescription.CNIC_OF_DIRECTORS_FRONT
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Cnic back,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_BACK,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_BACK)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_BACK),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_BACK)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_BACK,
                                                      subTitle: DocumentDescription.CNIC_OF_DIRECTORS_BACK
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Memorandum & Articles of association
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_MEMORANDUM_AND_ARTICLES,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_MEMORANDUM_AND_ARTICLES)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_MEMORANDUM_AND_ARTICLES),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_MEMORANDUM_AND_ARTICLES)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnMEMORANDUM_AND_ARTICLES,
                                                      subTitle: DocumentDescription.MEMORANDUM_AND_ARTICLES
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Certificate of incorporation
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_INCORPORATION_CERTIFICATE,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_INCORPORATION_CERTIFICATE)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_INCORPORATION_CERTIFICATE),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_INCORPORATION_CERTIFICATE)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnI_CERTIFICATE,
                                                      subTitle: DocumentDescription.CERTIFICATE_OF_INCORPORATION
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Board resolution duly signed
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_SIGNING_AUTHORITY_BOARD_RESOLUTION,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_SIGNING_AUTHORITY_BOARD_RESOLUTION)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_SIGNING_AUTHORITY_BOARD_RESOLUTION),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_SIGNING_AUTHORITY_BOARD_RESOLUTION)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnRESOLUTION_DULY_SIGNED,
                                                      subTitle: DocumentDescription.BOARD_RESOLUTION
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // List of directors
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_DIRECTORS_LIST_UNDER_COMPANIES_ORDINANCE,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_DIRECTORS_LIST_UNDER_COMPANIES_ORDINANCE)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_DIRECTORS_LIST_UNDER_COMPANIES_ORDINANCE),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_DIRECTORS_LIST_UNDER_COMPANIES_ORDINANCE)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  Utils.showTitleAndDescription(
                                                      RequiredDocs.btnLIST_OF_DIRECTORS_FORM_A_B,
                                                      DocumentDescription.LIST_OF_DIRECTORS_ON_FORM_A_B
                                                  ),
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Bank letter
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_DIRECTORS_BANK_LETTER,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_DIRECTORS_BANK_LETTER)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_DIRECTORS_BANK_LETTER),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_DIRECTORS_BANK_LETTER)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  Utils.showTitleAndDescription(
                                                      RequiredDocs.btnBANK_LETTER,
                                                      DocumentDescription.BANK_LETTER_WITH_DIRECTORS
                                                  ),
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Form 29 issues by the registrar
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_ALL_DIRECTORS_FROM_29,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_ALL_DIRECTORS_FROM_29)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_ALL_DIRECTORS_FROM_29),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_ALL_DIRECTORS_FROM_29)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  Utils.showTitleAndDescription(
                                                      RequiredDocs.btnLIST_OF_DIRECTORS_FORM_29,
                                                      DocumentDescription.LIST_OF_DIRECTORS_ON_FORM_29
                                                  ),
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                  ],
                                ),
                              ),
                              /**
                               * Cnic of all directors,
                               * Public notary of memorandum & Articles of association,
                               * Public notary of certificate of incorporation
                               * Board resolution duly signed by company secretary,
                               * Public notary of certificate of commencement of business(for public limited),
                               * List of directors on form-A/Form-B issued under companies ordinance,
                               * Bank letter mentioning the name of all directors,
                               * List of all directors on form 29 issued by the registrar
                               */
                              Visibility(
                                visible: showRequiredBox(5, null, null),
                                child: Column(
                                  children: [
                                    // Cnic of all directors
                                    // Cnic front,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_FRONT,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_FRONT)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_FRONT),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_FRONT)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_FRONT,
                                                      subTitle: DocumentDescription.CNIC_OF_DIRECTORS_FRONT
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Cnic back,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_BACK,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_BACK)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_BACK),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_BACK)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_BACK,
                                                      subTitle: DocumentDescription.CNIC_OF_DIRECTORS_BACK
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Memorandum & Articles of association
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_MEMORANDUM_AND_ARTICLES,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_MEMORANDUM_AND_ARTICLES)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_MEMORANDUM_AND_ARTICLES),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_MEMORANDUM_AND_ARTICLES)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnMEMORANDUM_AND_ARTICLES,
                                                      subTitle: DocumentDescription.MEMORANDUM_AND_ARTICLES
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Certificate of incorporation
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_INCORPORATION_CERTIFICATE,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_INCORPORATION_CERTIFICATE)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_INCORPORATION_CERTIFICATE),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_INCORPORATION_CERTIFICATE)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnI_CERTIFICATE,
                                                      subTitle: DocumentDescription.CERTIFICATE_OF_INCORPORATION
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Board resolution duly signed
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_SIGNING_AUTHORITY_BOARD_RESOLUTION,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_SIGNING_AUTHORITY_BOARD_RESOLUTION)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_SIGNING_AUTHORITY_BOARD_RESOLUTION),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_SIGNING_AUTHORITY_BOARD_RESOLUTION)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnRESOLUTION_DULY_SIGNED,
                                                      subTitle: DocumentDescription.BOARD_RESOLUTION
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Public notary of certificate of commencement of business
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                            onTap: () {
                                              selectAny(
                                                  DocKeys.V2_DOC_COMMENCEMENT_CERTIFICATE,
                                                  widget.apiKey,
                                                  widget.businessDetails.businessId
                                              );
                                            },
                                            child: drawDottedBox(
                                                child: Row(
                                                  children: [
                                                    Container(
                                                      width: Layout.uploadLogoW,
                                                      height: MediaQuery.of(context).size.height,
                                                      child: new Card(
                                                        elevation: 0,
                                                        color: Colors.transparent,
                                                        child: Stack(
                                                          children: [
                                                            Center(
                                                                child: showPreviewOfUploadedFile(DocKeys.V2_DOC_COMMENCEMENT_CERTIFICATE)
                                                            ),
                                                            // Logo visibility of cross to remove image.
                                                            Visibility(
                                                              visible: checkCrossVisibility(DocKeys.V2_DOC_COMMENCEMENT_CERTIFICATE),
                                                              child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_COMMENCEMENT_CERTIFICATE)),
                                                            ),
                                                          ],
                                                        ),
                                                      ),
                                                    ),
                                                    boxDescription(
                                                        title: RequiredDocs.btnC_CERTIFICATE,
                                                        subTitle: DocumentDescription.CERTIFICATE_OF_COMMENCEMENT
                                                    )
                                                  ],
                                                )
                                            )
                                        )
                                    ),
                                    // List of directors
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_DIRECTORS_LIST_UNDER_COMPANIES_ORDINANCE,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_DIRECTORS_LIST_UNDER_COMPANIES_ORDINANCE)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_DIRECTORS_LIST_UNDER_COMPANIES_ORDINANCE),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_DIRECTORS_LIST_UNDER_COMPANIES_ORDINANCE)),
                                                          )
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  Utils.showTitleAndDescription(
                                                      RequiredDocs.btnLIST_OF_DIRECTORS_FORM_A_B,
                                                      DocumentDescription.LIST_OF_DIRECTORS_ON_FORM_A_B
                                                  ),
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Bank letter
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_DIRECTORS_BANK_LETTER,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_DIRECTORS_BANK_LETTER)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_DIRECTORS_BANK_LETTER),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_DIRECTORS_BANK_LETTER)),
                                                          )
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  Utils.showTitleAndDescription(
                                                      RequiredDocs.btnBANK_LETTER,
                                                      DocumentDescription.BANK_LETTER_WITH_DIRECTORS
                                                  ),
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Form 29 issues by the registrar
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_ALL_DIRECTORS_FROM_29,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_ALL_DIRECTORS_FROM_29)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_ALL_DIRECTORS_FROM_29),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_ALL_DIRECTORS_FROM_29)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  Utils.showTitleAndDescription(
                                                      RequiredDocs.btnLIST_OF_DIRECTORS_FORM_29,
                                                      DocumentDescription.LIST_OF_DIRECTORS_ON_FORM_29
                                                  ),
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                  ],
                                ),
                              ),
                              /**
                               * Cnic,
                               * Copy of by-laws and rules/Certificate copy of instrument of trust/trust Deed,
                               * Resolution duly signed by all trustees,
                               * Authority letter,
                               * Registration certificate,
                               * Bank letter mentioning the names of all trustees,
                               * List of all trustees and
                               * Scan copy of last month of utility bills
                               */
                              Visibility(
                                visible: showRequiredBox(6, null, null),
                                child: Column(
                                  children: [
                                    // Cnic front,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_FRONT,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_FRONT)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_FRONT),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_FRONT)),
                                                          )
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_FRONT,
                                                      subTitle: DocumentDescription.CNIC_FRONT_NGO_CLUB_TRUST
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Cnic back,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_BACK,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_BACK)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_BACK),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_BACK)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_BACK,
                                                      subTitle: DocumentDescription.CNIC_BACK_NGO_CLUB_TRUST
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // By-laws and rules
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_BY_LAWS_AND_RULES,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_BY_LAWS_AND_RULES)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_BY_LAWS_AND_RULES),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_BY_LAWS_AND_RULES)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnLAW_RULES,
                                                      subTitle: DocumentDescription.BY_LAWS_AND_RULES
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Resolution duly signed
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_RESOLUTION,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_RESOLUTION)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_RESOLUTION),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_RESOLUTION)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnRESOLUTION,
                                                      subTitle: DocumentDescription.RESOLUTION_DULY_SIGNED
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Consent letter
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CONSENT_OR_AUTHORITY_LETTER,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CONSENT_OR_AUTHORITY_LETTER)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CONSENT_OR_AUTHORITY_LETTER),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CONSENT_OR_AUTHORITY_LETTER)),
                                                          )
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnAUTHORITY_LETTER,
                                                      subTitle: DocumentDescription.AUTHORITY_LETTER
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Upload Registration certificate
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_REGISTARTION_CERTIFICATE,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_REGISTARTION_CERTIFICATE)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_REGISTARTION_CERTIFICATE),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_REGISTARTION_CERTIFICATE)),
                                                          )
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnR_CERTIFICATE,
                                                      subTitle: DocumentDescription.R_CERTIFICATE_SHORT
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Bank letter
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                            onTap: () {
                                              selectAny(
                                                  DocKeys.V2_DOC_ALL_TRUSTEES_BANK_LETTER,
                                                  widget.apiKey,
                                                  widget.businessDetails.businessId
                                              );
                                            },
                                            child: drawDottedBox(
                                                child: Row(
                                                  children: [
                                                    Container(
                                                      width: Layout.uploadLogoW,
                                                      height: MediaQuery.of(context).size.height,
                                                      child: new Card(
                                                        elevation: 0,
                                                        color: Colors.transparent,
                                                        child: Stack(
                                                          children: [
                                                            Center(
                                                                child: showPreviewOfUploadedFile(DocKeys.V2_DOC_ALL_TRUSTEES_BANK_LETTER)
                                                            ),
                                                            // Logo visibility of cross to remove image.
                                                            Visibility(
                                                              visible: checkCrossVisibility(DocKeys.V2_DOC_ALL_TRUSTEES_BANK_LETTER),
                                                              child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_ALL_TRUSTEES_BANK_LETTER)),
                                                            )
                                                          ],
                                                        ),
                                                      ),
                                                    ),
                                                    boxDescription(
                                                        title: RequiredDocs.btnBANK_LETTER,
                                                        subTitle: DocumentDescription.BANK_LETTER
                                                    )
                                                  ],
                                                )
                                            )
                                        )
                                    ),
                                    // List of all trustees
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_LIST_OF_TRUSTEES_OR_MEMBERS,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_LIST_OF_TRUSTEES_OR_MEMBERS)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_LIST_OF_TRUSTEES_OR_MEMBERS),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_LIST_OF_TRUSTEES_OR_MEMBERS)),
                                                          )
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnLIST_OF_TRUSTEES,
                                                      subTitle: DocumentDescription.LIST_OF_TRUSTEES
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Upload Utility bill
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_UTILITY_BILL,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: new Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_UTILITY_BILL)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_UTILITY_BILL),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_UTILITY_BILL)),
                                                          )
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnUTILITY_BILL,
                                                      subTitle: DocumentDescription.UTILITY_BILL
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                  ],
                                ),
                              ),

                              Visibility(
                                visible: showRequiredBox(7, null , null),
                                child: Column(
                                  children: [
                                    // Cnic front,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_FRONT,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_FRONT)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_FRONT),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_FRONT)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_FRONT,
                                                      subTitle: DocumentDescription.CNIC_FRONT
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Cnic back,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_CNIC_BACK,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_CNIC_BACK)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_CNIC_BACK),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_CNIC_BACK)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnCNIC_BACK,
                                                      subTitle: DocumentDescription.CNIC_BACK
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    //IncomeProof
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_INCOME_PROOF,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_INCOME_PROOF)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_INCOME_PROOF),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_INCOME_PROOF)),
                                                          ),
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnPROOF_SOURCE_INCOME,
                                                      subTitle: DocumentDescription.PROOF_OF_SOURCE_OF_INCOME
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    ),
                                    // Account Maintenance certificate,
                                    Padding(
                                        padding: PaddingDetails.BOX,
                                        child: GestureDetector(
                                          onTap: () {
                                            selectAny(
                                                DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE,
                                                widget.apiKey,
                                                widget.businessDetails.businessId
                                            );
                                          },
                                          child: drawDottedBox(
                                              child: Row(
                                                children: [
                                                  Container(
                                                    width: Layout.uploadLogoW,
                                                    height: MediaQuery.of(context).size.height,
                                                    child: Card(
                                                      elevation: 0,
                                                      color: Colors.transparent,
                                                      child: Stack(
                                                        children: [
                                                          Center(
                                                              child: showPreviewOfUploadedFile(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE)
                                                          ),
                                                          // Logo visibility of cross to remove image.
                                                          Visibility(
                                                            visible: checkCrossVisibility(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE),
                                                            child: Utils.crossImage(() => _removePicture(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE)),
                                                          ),
                                                          /*// Logo of upload image.
                                                        Visibility(
                                                            visible: !checkCrossVisibility(DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE),
                                                            child: Utils.uploadIcon()
                                                        )*/
                                                        ],
                                                      ),
                                                    ),
                                                  ),
                                                  boxDescription(
                                                      title: RequiredDocs.btnM_CERTIFICATE,
                                                      subTitle: DocumentDescription.M_CERTIFICATE
                                                  )
                                                ],
                                              )
                                          ),
                                        )
                                    )
                                  ],
                                ),
                              ),
                              // Terms and conditions
                              Padding(
                                padding: const EdgeInsets.only(top: Layout.textViewTop),
                                child: Row(
                                  children: [
                                    Checkbox(
                                      value: _termsAndConditions,
                                      onChanged: (value) {
                                        setState(() {
                                          _termsAndConditions = value;
                                        });
                                      },
                                    ),
                                    RichText(
                                      text: TextSpan(
                                          style: TextStyle(
                                              color: Colors.grey,
                                              fontSize: Layout.dp18
                                          ),
                                          children: <TextSpan>[
                                            TextSpan(text: "Accept "),
                                            TextSpan(
                                                text: TextOf.TERMS_AND_CONDITIONS,
                                                style: TextStyle(color: Colors.blue),
                                                recognizer: TapGestureRecognizer()
                                                  ..onTap = () {
                                                    _launchURL();
                                                  }
                                            )
                                          ]
                                      ),
                                    )
                                  ],
                                ),
                              )
                            ],
                          ),
                        ),
                      )
                  ),
                  // submit button
                  Padding(
                    padding: PaddingDetails.NOTE,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: [
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            Utils.getHelplineMark(),
                            Utils.getLogoMark(),
                          ],
                        ),
                        SizedBox(
                          width: Layout.BTN_WIDTH,
                          child: ElevatedButton(
                            onPressed: () async {
                              if (documentsMap.length < count) {
                                Utils.showSnackBar(_scaffoldStateKey.currentState,
                                    "Please provide required document(s)");
                                return;
                              }
                              if (_termsAndConditions) {
                                widget.businessDetails.isSubmitting = true;
                                Utils.showLoaderDialogWithText(context, "Submitting...");
                                Utils.onBoardBusiness(widget.apiKey, widget.businessDetails).then((value) {
                                  Navigator.of(context).pop(false);
                                  _showResponseDialog(General.showSubmitResponse(widget.businessDetails.businessId));
                                }).timeout(
                                    Duration(seconds: Utils.timeOutSec),
                                    onTimeout: () {
                                      Navigator.of(context).pop(false);
                                      Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
                                      return;
                                    }
                                ).catchError((onError) {
                                  print(onError);
                                  Navigator.of(context).pop(false);
                                  Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
                                });
                              } else {
                                Utils.showSnackBar(_scaffoldStateKey.currentState, "Please accept Terms and Conditions");
                              }
                            },
                            child: Text(
                              ButtonTags.SUBMIT,
                            ),
                          ),
                        )
                      ],
                    ),
                  )
                ],
              ),
            )
        ),
        onWillPop: () {
          Navigator.of(context).pop(false);
          return Future.value(false);
        }
    );
  }
  Future<void> uploadImage(File file, String fieldId, String apiKey, String businessId, ImageSource source) async {
    if (!Utils.isFileExtensionSupported(file)) {
      Utils.showSnackBar(_scaffoldStateKey.currentState, "Invalid file type");
      return;
    }
    if (Utils.isNotGreaterThen5MB(file)) {
      bool result;
      if (source == ImageSource.camera) {
        result = true;
      } else {
        result = await Utils.showPreview(context, file, false);
      }
      if (result != null && result) {
        Utils.showLoaderDialogWithText(context, "Uploading...");
        await Utils.upload(widget.apiKey, file, fieldId, businessId).then((response) => {
          if (response != null) {
            if (response.responseCode == 0) {
              documentsMap[fieldId] = response.responseData,
              requiredDocs[fieldId] = true
            },
            Navigator.of(context).pop(false),
            Utils.showSnackBar(_scaffoldStateKey.currentState, response.responseMessage)
          }
        }).timeout(
            Duration(seconds: Utils.timeOutSec),
            onTimeout: () {
              Navigator.of(context).pop(false);
              Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
              return;
            }
        ).catchError((onError) {
          print(onError);
          Navigator.of(context).pop(false);
          Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
        });
      }
      else return;
    } else {
      Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.FILE_SIZE);
      return;
    }
    setState(() { });
  }
  void _removePicture(String key) async {
    Utils.showLoaderDialogWithText(context, "Removing...");
    if (widget.businessDetails.businessId != null && widget.businessDetails.businessId.isNotEmpty) {
      if (documentsMap.containsKey(key)) {
        RemoveImage removeImage = RemoveImage(
            key: key,
            imageUrl: documentsMap[key],
            businessId: widget.businessDetails.businessId
        );
        await Utils.getClient(widget.apiKey).removeImage(removeImage).then((response) {
          if (response != null) {
            ImageResponse imageResponse = ImageResponse.fromJson(
                jsonDecode(response), key);
            if (imageResponse.responseCode == 0) {
              setState(() {
                if (documentsMap.containsKey(key))
                  documentsMap.remove(key);
                requiredDocs[key] = false;
              });
            }
            Navigator.of(context).pop(false);
            Utils.showSnackBar(
                _scaffoldStateKey.currentState, imageResponse.responseMessage);
          }
        }).timeout(
            Duration(seconds: Utils.timeOutSec),
            onTimeout: () {
              Navigator.of(context).pop(false);
              Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
            }
        ).catchError((onError) {
          print(onError);
          Navigator.of(context).pop(false);
          Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
        });
      }
    } else {
      Navigator.of(context).pop(false);
      setState(() {
        if (documentsMap.containsKey(key))
          documentsMap.remove(key);
        requiredDocs[key] = false;
      });
    }
  }
  Widget showImage(String key) {
    switch(key) {
      case JsonFieldKey.NTN: return _fetchOrSetImage(key);
      case JsonFieldKey.FRONT_CNIC: return _fetchOrSetImage(key);
      case JsonFieldKey.BACK_CNIC: return _fetchOrSetImage(key);
      case JsonFieldKey.UTILITY_BILL: return _fetchOrSetImage(key);
      case JsonFieldKey.MAINTENANCE_CERTIFICATE: return _fetchOrSetImage(key);
      case JsonFieldKey.REGISTRATION_CERTIFICATE: return _fetchOrSetImage(key);
      case DocKeys.V2_DOC_ACC_MAINTENANCE_CERTIFICATE: return _fetchOrSetImage(key);
      default: return null;
    }
  }
  Widget _fetchOrSetImage(String key) {
    setState(() {
      if (documentsMap == null)
        return null;
      if (documentsMap.containsKey(key)) {
        String value = documentsMap[key];
        if (value.isNotEmpty) {
          if (value.contains("https"))
            return Image.network(value);
          else
            return Image.file(File(value), fit: BoxFit.fill);
        }
      }
    });
    return null;
  }
  void _launchURL() async {
    const url = 'https://foree.co/terms';
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  }
  selectAny(String fieldId, String apiKey, String businessId) {
    showModalBottomSheet(
        context: context,
        builder: (BuildContext context) {
          return Container(
            child: new Wrap(
              children: <Widget>[
                Visibility(
                  visible: Platform.isIOS,
                  child: new ListTile(
                      leading: Icon(Icons.file_copy, size: 30.0),
                      title: new Text('File', style: TextStyle(
                        fontSize: 18.0,
                        letterSpacing: 0.5,
                      )
                      ),
                      onTap: () {
                        Navigator.of(context).pop(false);
                        uploadFile(fieldId,apiKey,businessId);
                      }
                  ),
                ),
                new ListTile(
                  leading: new Icon(Icons.perm_media, size: 30.0),
                  title: new Text('Photo', style: TextStyle(
                    fontSize: 18.0,
                    letterSpacing: 0.5,
                  )
                  ),
                  onTap: () {
                    Navigator.of(context).pop(false);
                    uploadFromGallery(fieldId,apiKey,businessId);
                  },
                ),
                new ListTile(
                  leading: new Icon(Icons.camera_alt, size: 30.0),
                  title: new Text('Camera', style: TextStyle(
                    fontSize: 18.0,
                    letterSpacing: 0.5,
                  )
                  ),
                  onTap: () {
                    Navigator.of(context).pop(false);
                    takePictureFromCamera(fieldId,apiKey,businessId);
                  },
                ),
              ],
            ),
          );
        }
    );
  }

  void uploadFile(String fieldId, String apiKey, String businessId) async {
    if (await Permission.storage.isPermanentlyDenied) {
      openAppSettings();
    } else {
      await Permission.storage.request().isGranted.then((value) async {
        if(value){
          await FilePicker.getFile(type: FileType.custom, allowedExtensions: ['doc','pdf']).then((file) {
            if(file == null) return;
            uploadImage(file, fieldId, apiKey, businessId, ImageSource.gallery);
          });
        }
      });
    }
  }

  void uploadFromGallery(String fieldId, String apiKey, String businessId) async {
    if (await Permission.storage.isPermanentlyDenied) {
      openAppSettings();
    }
    else{
      await Permission.storage.request().isGranted.then((value) async {
        if(value){
          if(Platform.isIOS){
            await ImagePicker.pickImage(source: ImageSource.gallery).then((file) {
              if(file == null) return;
              uploadImage(file, fieldId, apiKey, businessId, ImageSource.gallery);
            });
          }
          if(Platform.isAndroid){
            await FilePicker.getFile(type: FileType.custom, allowedExtensions: ['jpg','png', 'jpeg', 'pdf']).then((file) {
              if(file == null) return;
              uploadImage(file, fieldId, apiKey, businessId, ImageSource.gallery);
            });
          }
        }
      });
    }
  }

  void takePictureFromCamera(String fieldId, String apiKey, String businessId) async {
    if (await Permission.camera.isPermanentlyDenied) {
      openAppSettings();
    }
    else {
      await Permission.camera.request().isGranted.then((value) async {
        if (value) {
          await ImagePicker.pickImage(source: ImageSource.camera).then((file) {
            if (file == null) return;
            uploadImage(file, fieldId, apiKey, businessId, ImageSource.camera);
          });
        }
      });
    }
  }

  Future<void> _showResponseDialog(String message) async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false, // user must tap button!
      builder: (BuildContext context) {
        return AlertDialog(
          contentPadding: EdgeInsets.all(20.0),
          content:  Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                Text(message, style: TextStyle(fontSize: 18.0)),
                Container(
                  width: Layout.dp100,
                  margin: PaddingDetails.EditIconPadding,
                  child: ElevatedButton(
                    onPressed: () {
                      Navigator.of(context).pop();
                      _isSubmitted = true;
                      widget.callbackBusinessId(widget.businessDetails.businessId);
                      Navigator.of(context).pop([true, _isSubmitted]);
                    },
                    child: Text(
                      ButtonTags.OK,
                    ),
                  ),
                )
              ],
          )
          //content: Text(message, style: TextStyle(fontSize: 18.0)),
        );
      },
    );
  }
}