import 'dart:convert';
import 'dart:io';
import 'dart:math';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'BankAccountDetailsForm.dart';
import 'Constants.dart';
import 'Requests/BusinessDetailParams.dart';
import 'Responses/BanksResponse.dart';
import 'Responses/City.dart';
import 'Responses/Province.dart';
import 'RetrofitConfigForee.dart';
import 'Utilities/Utils.dart';

class BusinessLocation extends StatefulWidget {
  final apiKey;
  final UserDetails userDetails;
  final BusinessDetailParams businessDetails;
  final ValueSetter<String> callbackBusinessId;
  BusinessLocation({Key key, this.apiKey, this.userDetails,
    this.businessDetails, this.callbackBusinessId}) : super(key: key);
  @override
  _BusinessLocationState createState() => _BusinessLocationState();
}
class _BusinessLocationState extends State<BusinessLocation> {
  final _formKey = GlobalKey<FormState>();
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();

  Widget logo;
  Province _province;
  City _city;
  List<Bank> _banks;
  int cityId;
  int provinceId;
  AutovalidateMode _autoValidateMode;
  List<City> cities = [];

  @override
  void initState() {
    _autoValidateMode = AutovalidateMode.disabled;
    Utils.fetchBanks(widget.apiKey).then((value) => _banks = value);
    if (widget.businessDetails.city != null && widget.businessDetails.city.id != null) {
      cityId = widget.businessDetails.city.id - 1;
    } else {
      cityId = 15;
    }
    if (widget.businessDetails.province != null && widget.businessDetails.province.id != null) {
      provinceId = widget.businessDetails.province.id - 1;
    } else {
      provinceId = 0;
    }
    print(cityId);
    print(provinceId);
    _city = Utils.cities[cityId];
    _province = Utils.provinces[provinceId];
    cities = Utils.cities.where((element) => element.provinceId == _province.id).toList();
    cityId = 0;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        child: Scaffold(
          key: _scaffoldStateKey,
          appBar: AppBar(
            title: Text(Titles.BUSINESS_LOCATION),
            leading: logo,
            automaticallyImplyLeading: Platform.isIOS,
          ),
          body: Padding(
            padding: PaddingDetails.FORM,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Expanded(
                  child: SingleChildScrollView(
                    child: Form(
                      key: _formKey,
                      autovalidateMode: _autoValidateMode,
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          // Subtitle
                          Padding(
                            padding: PaddingDetails.FIELD_TOP,
                            child: Text(
                              SubTitle.BUSINESS_LOCATION_DESCRIPTION,
                              style: TextStyling.getHeaderStyle(),
                            ),
                          ),
                          // Provinces
                          Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: DropdownButtonFormField(
                                value: Utils.provinces[provinceId],
                                isExpanded: true,
                                items: Utils.provinces.map((province) =>
                                    DropdownMenuItem(
                                      child: Text(province.name),
                                      value: province,
                                    )
                                ).toList(),
                                validator: (value) {
                                  if (value == null)
                                    return Utils.showEnterFieldError("Province");
                                  else return null;
                                },
                                decoration: InputDecoration(
                                    border: OutlineInputBorder(),
                                    labelText: FormLabels.PROVINCE
                                ),
                                iconEnabledColor: CupertinoColors.darkBackgroundGray,
                                onTap: () => FocusScope.of(context)
                                    .requestFocus(FocusNode()),
                                onChanged: (value) {
                                  _province = (value as Province );
                                  cities = Utils.cities.where((element) => element.provinceId == _province.id).toList();
                                  cityId = 0;
                                },
                              )
                          ),
                          // Cities
                          Padding(
                              padding: PaddingDetails.FIELD_TOP,
                              child: DropdownButtonFormField(
                                value: cities[cityId],
                                isExpanded: true,
                                items: cities.map((city) => DropdownMenuItem(
                                  child: Text(city.name),
                                  value: city,
                                )).toList(),
                                validator: (value) {
                                  if (value == null)
                                    return Utils.showEnterFieldError("City");
                                  else return null;
                                },
                                decoration: InputDecoration(
                                    border: OutlineInputBorder(),
                                    labelText: FormLabels.CITY
                                ),
                                iconEnabledColor: CupertinoColors.darkBackgroundGray,
                                onTap: () => FocusScope.of(context)
                                    .requestFocus(FocusNode()),
                                onChanged: (value) {_city = (value as City);},
                              )
                          ),
                          // Area
                          Padding(
                            padding: PaddingDetails.FIELD_TOP,
                            child: TextFormField(
                              // controller: _areaController,
                              decoration: InputDecoration(
                                  border: const OutlineInputBorder(),
                                  labelText: FormLabels.AREA
                              ),
                              validator: (value) {
                                if (value == null) {
                                  return Utils.showEnterFieldError("Area");
                                } else if (value.length < LengthLimit.MIN_LENGTH_3) {
                                  return ErrorFields.MIN_LENGTH_ERROR_3;
                                } else if (value.length > LengthLimit.MAX_LENGTH_500) {
                                  return ErrorFields.MAX_LENGTH_ERROR_500;
                                } else {
                                  return null;
                                }
                              },
                              initialValue: widget.businessDetails.area,
                              onChanged: (value) => widget.businessDetails.area = value,
                            ),
                          ),
                          // Business address
                          Padding(
                            padding: PaddingDetails.FIELD_TOP,
                            child: TextFormField(
                              // controller: _businessAddressController,
                              decoration: InputDecoration(
                                  border: const OutlineInputBorder(),
                                  labelText: FormLabels.BUSINESS_ADDRESS
                              ),
                              validator: (value) {
                                if (value == null) {
                                  return Utils.showEnterFieldError("Business address");
                                } else if (value.length < LengthLimit.MIN_LENGTH_3) {
                                  return ErrorFields.MIN_LENGTH_ERROR_3;
                                } else if (value.length > LengthLimit.MAX_LENGTH_500) {
                                  return ErrorFields.MAX_LENGTH_ERROR_500;
                                } else {
                                  return null;
                                }
                              },
                              initialValue: widget.businessDetails.businessAddress,
                              onChanged: (value) => widget.businessDetails.businessAddress = value,
                            ),
                          ),
                        ],
                      ),
                    ),
                  ),
                ),
                // Buttons
                Padding(
                  padding: PaddingDetails.NOTE,
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.end,
                    children: [
                      Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: [
                          Utils.getHelplineMark(),
                          Utils.getLogoMark(),
                        ],
                      ),
                      Row(
                        children: [
                          // Button save
                          Expanded(
                            flex: 2,
                            child: Padding(
                              padding: PaddingDetails.BTN_RIGHT,
                              child: SizedBox(
                                width: Layout.BTN_WIDTH,
                                child: ElevatedButton(
                                  onPressed: () {
                                    onBoardBusiness(
                                        isSubmitting: false,
                                        moveToNext: false
                                    );
                                    FocusScope.of(context).unfocus();
                                  },
                                  child: Text(
                                    ButtonTags.SAVE,
                                    style: TextStyle(
                                      color: Theme.of(context).primaryColor,
                                    ),
                                  ),
                                  style: ButtonStyle(
                                      backgroundColor: MaterialStateColor.resolveWith(
                                              (states) => Colors.white
                                      )
                                  ),
                                ),
                              ),
                            ),
                          ),
                          // Button Next
                          Expanded(
                            flex: 2,
                            child: Padding(
                              padding: PaddingDetails.BTN_LEFT,
                              child: SizedBox(
                                width: Layout.BTN_WIDTH,
                                child: ElevatedButton(
                                    onPressed: () async {
                                      if (_formKey.currentState.validate()) {
                                        onBoardBusiness(
                                            isSubmitting: false,
                                            moveToNext: true
                                        );
                                      } else {
                                        setState(() {
                                          _autoValidateMode = AutovalidateMode.always;
                                        });
                                      }
                                    },
                                    child: Text(
                                      ButtonTags.NEXT,
                                    )
                                ),
                              ),
                            ),
                          )
                        ],
                      )
                    ],
                  ),
                )
              ],
            ),
          ),
        ),
        onWillPop: () {
          Navigator.of(context).pop(false);
          return Future.value(false);
        }
    );
  }
  void additionalBusinessDetails(BusinessDetailParams businessDetails) {
    Navigator.push(context, MaterialPageRoute(
        builder: (context) =>
            AccountDetailsForm(
              apiKey: widget.apiKey,
              banks: _banks,
              userDetails: UserDetails(),
              businessDetails: businessDetails,
              callbackBusinessId: widget.callbackBusinessId,
            )
    )).then((value) {
      if (value != null && value[0] && value[1])
        Navigator.of(context).pop(value);
    });
  }
  void onBoardBusiness({bool isSubmitting, bool moveToNext}) async {
    widget.businessDetails.isSubmitting = isSubmitting;
    widget.businessDetails.province = _province;
    widget.businessDetails.city = _city;
    // widget.businessDetails.area = _areaController.value.text;
    // widget.businessDetails.businessAddress = _businessAddressController.value.text;
    Utils.apiCallsHandler(context, _scaffoldStateKey.currentState, () async {
      moveToNext ? Utils.showLoaderDialogWithText(context, "Please wait...")
          : Utils.showLoaderDialogWithText(context, "Saving...");
      await Utils.onBoardBusiness(widget.apiKey, widget.businessDetails).then((response) {
        if (response != null) {
          var data = json.decode(response);
          var responseCode = data['response_code'];
          var responseMessage = data['response_message'];
          if (responseCode == 0) {
            Navigator.of(context).pop(false);
            if (!moveToNext)
              widget.callbackBusinessId(widget.businessDetails.businessId);
            moveToNext ? additionalBusinessDetails(widget.businessDetails) : Utils.showSnackBar(
                _scaffoldStateKey.currentState, General.showSaveResponse2());
          } else {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, responseMessage);
          }
        }
      }).timeout(
          Duration(seconds: Utils.timeOutSec),
          onTimeout: () {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
          }
      ).catchError((onError) {
        print(onError);
        Navigator.of(context).pop(false);
        Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
      });
    });
  }
}