import 'dart:convert';
import 'dart:io';

import 'package:checkbox_grouped/checkbox_grouped.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'BusinessLocation.dart';
import 'Constants.dart';
import 'Requests/BusinessDetailParams.dart';
import 'Responses/FeeBorne.dart';
import 'Responses/InstrumentType.dart';
import 'RetrofitConfigForee.dart';
import 'SupportedInstruments.dart';
import 'Utilities/Utils.dart';

class PaymentMethod extends StatefulWidget {
  final String apiKey;
  final UserDetails userDetails;
  final Map<String, String> data;
  final BusinessDetailParams businessDetails;
  final ValueSetter<String> callbackBusinessId;
  PaymentMethod({Key key, @required this.apiKey, this.data, this.userDetails,
    this.businessDetails, this.callbackBusinessId}) : super(key: key);
  @override
  _PaymentMethodState createState() => _PaymentMethodState();
}
class _PaymentMethodState extends State<PaymentMethod> {
  final _formKey = GlobalKey<FormState>();
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();
  final GroupController _paymentMethodController = GroupController(isMultipleSelection: true);

  Widget logo;
  List<Widget> _paymentChargers;
  List<Map<String, dynamic>> _supportedInstruments;
  List<InstrumentType> _instrumentTypes;
  static Map<int, FeeBorne> _selectedFeeBorne = Map();
  List<dynamic> test;

  @override
  void initState() {
    print("Business Settlement ${widget.businessDetails.settlementPeriod}");
    _paymentChargers = List();
    _supportedInstruments = List();
    _instrumentTypes = List();
    test = List();
    for (var type in Utils.instrumentType) {
      type.active = false;
      _instrumentTypes.add(type);
    }
    List<int> temp = List();
    for (Map<String, dynamic> obj in widget.businessDetails.supportedInstrument) {
      SupportedInstruments si = SupportedInstruments.fromJson(obj);
      if (si.instrumentType.active) {
        _instrumentTypes[si.instrumentType.id - 1].active = true;
        temp.add(si.instrumentType.id);
        _selectedFeeBorne[si.instrumentType.id] = si.feeBorne;
        _paymentChargers.add(PaymentChargesAction(
          key: Key("index_${si.instrumentType.id}"),
          index: si.instrumentType.id,
          title: si.instrumentType.caption ?? _instrumentTypes[si.instrumentType.id - 1].caption,
          feeBorneId: si.feeBorne.id - 1,
        ));
      }
    }
    _paymentMethodController.initSelectedItem = temp;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        child: Scaffold(
          key: _scaffoldStateKey,
          appBar: AppBar(
            title: Text(Titles.PAYMENT_METHOD),
            leading: logo,
            automaticallyImplyLeading: Platform.isIOS,
          ),
          body: Padding(
            padding: PaddingDetails.FORM,
            child: Form(
              key: _formKey,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  // Fields
                  Expanded(
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        // Heading
                        Padding(
                          padding: PaddingDetails.FIELD_TOP,
                          child: Text(
                            FormLabels.ACCEPT_PAYMENT_FROM,
                            style: TextStyle(
                                fontSize: 16.0,
                                fontWeight: FontWeight.w600
                            ),
                          ),
                        ),
                        // Payment methods
                        SimpleGroupedCheckbox<int>(
                          controller: _paymentMethodController,
                          itemsTitle: Utils.instrumentType.map((type) => type.caption).toList(),
                          values: Utils.instrumentType.map((type) => type.id).toList(),
                          activeColor: Theme.of(context).primaryColor,
                          checkFirstElement: false,
                          isLeading: true,
                          onItemSelected: (value) {
                            _paymentChargers.clear();
                            setState(() {
                              for (int id in value) {
                                _paymentChargers.add(PaymentChargesAction(
                                  key: Key("index_$id"),
                                  index: id,
                                  title: Utils.instrumentType[(id - 1)].caption,
                                  feeBorneId: 0,
                                ));
                              }
                              for (InstrumentType type in Utils.instrumentType) {
                                if ((value as List).contains(type.id))
                                  _instrumentTypes[(type.id-1)].active = true;
                                else
                                  _instrumentTypes[(type.id-1)].active = false;
                              }
                            });
                          },
                        ),
                        Visibility(
                          visible: _paymentChargers.isNotEmpty,
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              // Heading
                              Padding(
                                padding: PaddingDetails.FIELD_TOP,
                                child: Text(
                                  FormLabels.DEDUCT_PAYMENT_CHARGES_FROM,
                                  style: TextStyle(
                                      fontSize: 16,
                                      fontWeight: FontWeight.w600
                                  ),
                                ),
                              ),
                              ListBody(
                                children: _paymentChargers,
                              )
                            ],
                          ),
                        )
                      ],
                    ),
                  ),
                  // Buttons
                  Padding(
                    padding: PaddingDetails.NOTE,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Container(
                          padding: PaddingDetails.PAYMENT_NOTE_PADDING,
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Text("Your payments get settled to your account in,"),
                              Row(
                                children: [
                                  Text(HelpText.BANKACCOUNT_PAYMENT_NOTE),
                                  Text("T+"+widget.businessDetails.settlementPeriod[1].settlement_days.toString(), style: TextStyle(fontWeight: FontWeight.bold)),
                                  Text(" working day"),
                                ],
                              ),
                              Row(
                                children: [
                                  Text(HelpText.CARD_PAYMENT_NOTE),
                                  Text("T+"+widget.businessDetails.settlementPeriod[0].settlement_days.toString(), style: TextStyle(fontWeight: FontWeight.bold)),
                                  Text(" working days"),
                                ],
                              ),
                              // Text(
                              //   widget.businessDetails.settlementPeriod[1].settlement_days > 1 ?
                              //   HelpText.BANKACCOUNT_PAYMENT_NOTE_TIME + widget.businessDetails.settlementPeriod[1].settlement_days.toString() + " working days"  :
                              //   HelpText.BANKACCOUNT_PAYMENT_NOTE + widget.businessDetails.settlementPeriod[1].settlement_days.toString() + " working days", style: TextStyle( fontStyle: FontStyle.italic)),
                              // Text(
                              //     widget.businessDetails.settlementPeriod[0].settlement_days > 1 ?
                              //     HelpText.CARD_PAYMENT_NOTE_TIME + widget.businessDetails.settlementPeriod[0].settlement_days.toString() + " working days"  :
                              //     HelpText.CARD_PAYMENT_NOTE + widget.businessDetails.settlementPeriod[0].settlement_days.toString() + " working days", style: TextStyle( fontStyle: FontStyle.italic)),
                              Text("(T is the date of payment)"),
                            ],
                          ),
                        ),
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            Utils.getHelplineMark(),
                            Utils.getLogoMark(),
                          ],
                        ),
                        Row(
                          children: [
                            // Button save
                            Expanded(
                              flex: 2,
                              child: Padding(
                                padding: PaddingDetails.BTN_RIGHT,
                                child: SizedBox(
                                  width: Layout.BTN_WIDTH,
                                  child: ElevatedButton(
                                    onPressed: () async {
                                      if (_paymentChargers.isNotEmpty) {
                                        _supportedInstruments.clear();
                                        for (var type in _instrumentTypes) {
                                          _supportedInstruments.add(
                                              SupportedInstruments(
                                                  feeBorne: (_selectedFeeBorne
                                                      .containsKey(type.id) ?
                                                  _selectedFeeBorne[type.id] :
                                                  Utils.feeBorne[0]
                                                  ),
                                                  instrumentType: type
                                              ).toJson()
                                          );
                                        }
                                        onBoardBusiness(
                                            isSubmitting: false,
                                            moveToNext: false
                                        );
                                        FocusScope.of(context).unfocus();
                                      } else {
                                        Utils.showSnackBar(_scaffoldStateKey
                                            .currentState, "Please select payment method");
                                      }
                                    },
                                    child: Text(
                                      ButtonTags.SAVE,
                                      style: TextStyle(
                                        color: Theme.of(context).primaryColor,
                                      ),
                                    ),
                                    style: ButtonStyle(backgroundColor:
                                    MaterialStateColor.resolveWith(
                                            (states) => Colors.white
                                    )
                                    ),
                                  ),
                                ),
                              ),
                            ),
                            // Button Next
                            Expanded(
                              flex: 2,
                              child: Padding(
                                padding: PaddingDetails.BTN_LEFT,
                                child: SizedBox(
                                  width: Layout.BTN_WIDTH,
                                  child: ElevatedButton(
                                      onPressed: () async {
                                        if (_paymentChargers.isNotEmpty) {
                                          _supportedInstruments.clear();
                                          for (var type in _instrumentTypes) {
                                            _supportedInstruments.add(
                                                SupportedInstruments(
                                                    feeBorne: (_selectedFeeBorne
                                                        .containsKey(type.id) ?
                                                    _selectedFeeBorne[type.id] :
                                                    Utils.feeBorne[0]
                                                    ),
                                                    instrumentType: type
                                                ).toJson()
                                            );
                                          }
                                          onBoardBusiness(
                                              isSubmitting: false,
                                              moveToNext: true
                                          );
                                        } else {
                                          Utils.showSnackBar(_scaffoldStateKey
                                              .currentState, "Please select payment method");
                                        }
                                      },
                                      child: Text(
                                        ButtonTags.NEXT,
                                      )
                                  ),
                                ),
                              ),
                            )
                          ],
                        )
                      ],
                    ),
                  )
                ],
              ),
            ),
          ),
        ),
        onWillPop: () {
          Navigator.of(context).pop(false);
          return Future.value(false);
        }
    );
  }
  void businessLocationFlow(BusinessDetailParams businessDetails) {
    Navigator.push(context, MaterialPageRoute(
        builder: (context) =>
            BusinessLocation(
              apiKey: widget.apiKey,
              userDetails: widget.userDetails,
              businessDetails: businessDetails,
              callbackBusinessId: widget.callbackBusinessId,
            )
    )).then((value) {
      if (value != null && value[0] && value[1])
        Navigator.of(context).pop(value);
    });
  }
  void onBoardBusiness({bool isSubmitting, bool moveToNext}) async {
    widget.businessDetails.isSubmitting = isSubmitting;
    widget.businessDetails.supportedInstrument = _supportedInstruments;
    print(widget.businessDetails.toJson());
    Utils.apiCallsHandler(context, _scaffoldStateKey.currentState, () async {
      moveToNext ? Utils.showLoaderDialogWithText(context, "Please wait...")
          : Utils.showLoaderDialogWithText(context, "Saving...");
      await Utils.onBoardBusiness(widget.apiKey, widget.businessDetails).then((response) {
        if (response != null) {
          var data = json.decode(response);
          var responseCode = data['response_code'];
          var responseMessage = data['response_message'];
          if (responseCode == 0) {
            Navigator.of(context).pop(false);
            if (!moveToNext)
              widget.callbackBusinessId(widget.businessDetails.businessId);
            moveToNext ? businessLocationFlow(widget.businessDetails) : Utils.showSnackBar(
                _scaffoldStateKey.currentState, General.showSaveResponse2());
          } else {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, responseMessage);
          }
        }
      }).timeout(
          Duration(seconds: Utils.timeOutSec),
          onTimeout: () {
            Navigator.of(context).pop(false);
            Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
          }
      ).catchError((onError) {
        print(onError);
        Navigator.of(context).pop(false);
        Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
      });
    });
  }
}

class PaymentChargesAction extends StatefulWidget {
  final int index;
  final String title;
  final int feeBorneId;
  PaymentChargesAction({ Key key, this.title, this.index, this.feeBorneId}) : super(key: key);
  @override
  State<StatefulWidget> createState() {
    return _PaymentChargersActionState();
  }
}
class _PaymentChargersActionState extends State<PaymentChargesAction> {
  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: PaddingDetails.TextFieldPadding,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(widget.title,
            style: TextStyle(
                fontSize: 16
            ),
          ),
          Padding(
            padding: EdgeInsets.only(top: 5.0),
            child: DropdownButtonFormField(
              value: (Utils.feeBorne[widget.feeBorneId]),
              items: Utils.feeBorne.map((feeBorne) {
                return DropdownMenuItem<dynamic>(
                  value: feeBorne,
                  child: Text(feeBorne.caption),
                );
              }).toList(),
              validator: (value) {
                if (value == null)
                  return Utils.showEnterFieldError("Type of Business");
                else return null;
              },
              iconEnabledColor: CupertinoColors.darkBackgroundGray,
              decoration: InputDecoration(
                  border: OutlineInputBorder(),
                  prefixText: "From "
              ),
              onTap: () => FocusScope.of(context).requestFocus(FocusNode()),
              onChanged: (value) {
                _PaymentMethodState._selectedFeeBorne[widget.index] = (value as FeeBorne);
              },
            ),
          )
        ],
      ),
    );
  }
}