import 'package:businessonboarding/src/Responses/City.dart';
import 'package:businessonboarding/src/Responses/Province.dart';
import 'package:json_annotation/json_annotation.dart';

import '../Constants.dart';
import '../RetrofitConfigForee.dart';

@JsonSerializable()
class BusinessDetailParams {
  String userId;
  bool isSubmitting;
  String businessId;
  // First page
  String businessName;
  String businessDescription;
  String emailAddress;
  String mobileNumber;
  BusinessType businessType;
  BusinessCategories businessCategories;
  // Second page
  String nameOfBusinessOwner;
  String cnicOfBusinessOwner;
  String ntn;
  String officialWebsite;
  String expectedMonthlyTurnOver;
  List<dynamic> partnerDetails;
  // Third page
  List<dynamic> supportedInstrument;
  // Forth page
  Province province;
  City city;
  String area;
  String businessAddress;
  // Fifth page
  BankAccountDetails bankAccountDetails;
  // Documents page
  Documents documents;

  //Settlement_Period
  List<dynamic> settlementPeriod;

  BusinessDetailParams({ this.userId, this.isSubmitting, this.businessId,
    this.businessName, this.businessDescription, this.emailAddress, this.mobileNumber,
    this.businessType, this.businessCategories, this.nameOfBusinessOwner, this.cnicOfBusinessOwner,
    this.ntn, this.officialWebsite, this.expectedMonthlyTurnOver, this.supportedInstrument,
    this.province, this.city, this.area, this.businessAddress, this.bankAccountDetails,
    this.documents, this.partnerDetails, this.settlementPeriod});

  factory BusinessDetailParams.fromJson(Map<String, dynamic> json) => _$BusinessDetailParamsFromJson(json);
  Map<String, dynamic> toJson() => _$BusinessDetailParamsToJson(this);
}

BusinessDetailParams _$BusinessDetailParamsFromJson(Map<String, dynamic> json) {
  print("JSON DATA $json");
  final categories = json[JsonFieldKey.BUSINESS_CATEGORY];
  final entities = json[JsonFieldKey.BUSINESS_LEGAL_ENTITY];
  final rawList = json['linked_banks'] as List;
  final province = json['province'];
  final city = json['city'];
  final supportedInstruments = json['supported_instruments'] as List;
  final documents = json[JsonFieldKey.DOCUMENTS];
  final bankAccountDetails = json[DataKey.BANK_ACCOUNT_DETAILS];
  final settlementPeriod = json['settlement_period'] as List;
  return BusinessDetailParams(
    userId: json["user_id"],
    isSubmitting: json["form_submitted"],
    businessId: json["business_id"],
    businessName: json[DataKey.BUSINESS_NAME],
    businessDescription: json[DataKey.BUSINESS_DESCRIPTION],
    emailAddress: json[DataKey.EMAIL_ADDRESS],
    mobileNumber: json[DataKey.MOBILE_NUMBER],
    businessType: (entities != null ? BusinessType.fromJson(entities) : BusinessType()),
    businessCategories: (categories != null ? BusinessCategories.fromJson(categories) : BusinessCategories()),
    nameOfBusinessOwner: json[DataKey.BUSINESS_OWNER_NAME] as String,
    cnicOfBusinessOwner: json[DataKey.BUSINESS_OWNER_CNIC_NUMBER] as String,
    officialWebsite: json[DataKey.WEBSITE_URL] as String,
    expectedMonthlyTurnOver: json[DataKey.EXPECTED_MONTHLY_TURNOVER] as String,
    supportedInstrument: (supportedInstruments != null ? supportedInstruments : List<dynamic>()),
    province: (province != null ? Province.fromJson(province) : Province()),
    city: (city != null ? City.fromJson(json["city"]) : City()),
    area: json[DataKey.AREA],
    businessAddress: json[DataKey.BUSINESS_ADDRESS],
    ntn: json[DataKey.NTN],
    bankAccountDetails: ((rawList != null && rawList.isNotEmpty) ? BankAccountDetails.fromJson(rawList[0]) : (bankAccountDetails == null ? BankAccountDetails() : BankAccountDetails.fromJson(bankAccountDetails))),
    documents: (documents != null ? Documents.fromJson(documents) : Documents()),
    partnerDetails: json[JsonFieldKey.PARTNERS_DETAILS] as List,
    settlementPeriod: settlementPeriod != null ? settlementPeriod.map((e) => SettlementPeriod.fromJson(e)).toList(): [],
  );
}

Map<String, dynamic> _$BusinessDetailParamsToJson(BusinessDetailParams instance) => <String, dynamic> {
  "user_id": instance.userId,
  "form_submitted": instance.isSubmitting,
  "business_id": instance.businessId,
  "name": instance.businessName,
  "description": instance.businessDescription,
  "email_address": instance.emailAddress,
  "phone_number": instance.mobileNumber,
  JsonFieldKey.BUSINESS_LEGAL_ENTITY: (instance.businessType == null ? Map<String, dynamic>() : instance.businessType.toJson()),
  JsonFieldKey.BUSINESS_CATEGORY: (instance.businessCategories == null ? Map<String, dynamic>() : instance.businessCategories.toJson()),
  JsonFieldKey.BUSINESS_OWNER_NAME: instance.nameOfBusinessOwner,
  JsonFieldKey.BUSINESS_OWNER_CNIC_NUMBER: instance.cnicOfBusinessOwner,
  JsonFieldKey.WEBSITE_URL: instance.officialWebsite,
  JsonFieldKey.EXPECTED_MONTHLY_TURNOVER: instance.expectedMonthlyTurnOver,
  "supported_instruments": instance.supportedInstrument,
  "province": (instance.province == null ? Province() : instance.province.toJson()),
  "city": (instance.city == null ? City() : instance.city.toJson()),
  "area": instance.area,
  "ntn": instance.ntn,
  "address": instance.businessAddress,
  JsonFieldKey.LINKED_BANKS: (instance.bankAccountDetails != null ? instance.bankAccountDetails.bankId == null ? List() : instance.bankAccountDetails.toJson() : List()),
  JsonFieldKey.PARTNERS_DETAILS: (instance.partnerDetails == null ? List() : instance.partnerDetails),
  "settlement_period": instance.settlementPeriod,
};