import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'BusinessDetails.dart';
import 'Constants.dart';
import 'RetrofitConfigForee.dart';

class UserBusinesses extends StatefulWidget {
  final String apiKey;
  final String userId;
  final UserDetails userDetails;
  final List businessList;
  final Map<String, String> data;
  UserBusinesses({Key key, this.apiKey, this.userId, this.businessList,
    this.data, this.userDetails}) : super(key: key);
  @override
  State<StatefulWidget> createState() {
    return _UserBusinessState();
  }
}
class _UserBusinessState extends State<UserBusinesses> {
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();

  Widget logo;
  List<Widget> _businesses = List();

  @override
  void initState() {
    for (var val in widget.businessList)
      _businesses.add(BusinessField(businessId: val));
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: _scaffoldStateKey,
      appBar: AppBar(
        title: Text(Titles.MY_BUSINESSES),
        leading: logo,
        automaticallyImplyLeading: Platform.isIOS,
      ),
      body: Column(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          Expanded(
            child: ListView.builder(
              itemCount: _businesses.length,
              itemBuilder: (context, index) {
                return _businesses[index];
              },
            ),
          ),
          Padding(
            padding: PaddingDetails.FORM,
            child: SizedBox(
              width: Layout.BTN_WIDTH,
              height: Layout.BTN_HEIGHT,
              child: ElevatedButton(
                  onPressed: () {
                    Navigator.push(context, MaterialPageRoute(
                        builder: (context) => BusinessDetails(
                          apiKey: widget.apiKey,
                          data: widget.data,
                        )
                    ));
                  },
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Padding(
                        padding: PaddingDetails.ICON,
                        child: Icon(
                          Icons.add_circle_outline,
                          color: Colors.white,
                        ),
                      ),
                      Text(
                          ButtonTags.Add_BUSINESS,
                          style: TextStyle(
                              fontSize: Layout.BTN_TEXT_SIZE
                          )
                      )
                    ],
                  )
              ),
            ),
          )
        ],
      ),
    );
  }
}

class BusinessField extends StatefulWidget {
  final String businessId;
  BusinessField({Key key, this.businessId}) : super(key: key);
  @override
  State<StatefulWidget> createState() {
    return _BusinessFieldState();
  }
}
class _BusinessFieldState extends State<BusinessField> {
  @override
  Widget build(BuildContext context) {
    return FlatButton(
      onPressed: () {
        print("Fetch business details against the following business id: ${widget.businessId}");
      },
      shape: UnderlineInputBorder(),
      child: Row(
        children: [
          Padding(
            padding: PaddingDetails.ICON,
            child: Icon(
              Icons.shopping_bag,
              color: Theme.of(context).primaryColor,
            ),
          ),
          Text(
            widget.businessId,
            style: TextStyle(
                fontSize: TextStyling.BUSINESSES_BTN_SIZE
            ),
          )
        ],
      ),
    );
  }
}