import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:businessonboarding/src/Requests/AcquireUserParams.dart';
import 'package:businessonboarding/src/Requests/BusinessDetailParams.dart';
import 'package:businessonboarding/src/Requests/BusinessIdParams.dart';
import 'package:businessonboarding/src/Requests/ForgotPasswordParams.dart';
import 'package:businessonboarding/src/Requests/LoginOrCreatePasswordParams.dart';
import 'package:businessonboarding/src/Requests/OtpParams.dart';
import 'package:businessonboarding/src/Requests/UpdateUserParams.dart';
import 'package:businessonboarding/src/Requests/VerifyParams.dart';
import 'package:businessonboarding/src/Responses/AppVersionInfoResponse.dart';
import 'package:businessonboarding/src/Responses/BanksResponse.dart';
import 'package:businessonboarding/src/Responses/City.dart';
import 'package:businessonboarding/src/Responses/FeeBorne.dart';
import 'package:businessonboarding/src/Responses/ForgotPassword.dart';
import 'package:businessonboarding/src/Responses/ImageResponse.dart';
import 'package:businessonboarding/src/Responses/InstrumentType.dart';
import 'package:businessonboarding/src/Responses/Otp.dart';
import 'package:businessonboarding/src/Responses/Province.dart';
import 'package:businessonboarding/src/Responses/User.dart';
import 'package:dio/dio.dart';
import 'package:dotted_border/dotted_border.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';

import '../RetrofitConfigForee.dart';
import '../Constants.dart';
import 'PreviewImage.dart';

class Utils {
  static ForeeRestClient client;
  static bool isTesting = false;
  static int id;
  static List<Bank> banks;
  static List<City> cities;
  static Map<int, String> feeBorne2;
  static List<Province> provinces;
  static List<FeeBorne> feeBorne;
  static List<InstrumentType> instrumentType;
  static int timeOutSec = 120;
  static String getImageBase64(String image) {
    final bytes = File(image).readAsBytesSync();
    return base64Encode(bytes);
  }
  static ForeeRestClient getClient(String apiKey) {
    if (client == null) {
      final dio = Dio(BaseOptions(contentType: "application/json"));
      dio.options.headers[RequestHeader.X_AGENT_KEY] = apiKey;
      client = ForeeRestClient(dio);
    }
    return client;
  }
  static Future<AppVersionInfoResponse> versionInfoCall(String apiKey,
      String appVersion) async {
    AppVersionInfoResponse info;
    if (apiKey != null && apiKey.isNotEmpty) {
      VersionInfo version = new VersionInfo();
      version.sdkPlatform = "flutter";
      version.sdkAppVersion = appVersion;
      await getClient(apiKey).getVersion(version).then((response) => {
        info = AppVersionInfoResponse.fromJson(jsonDecode(response)[ResponseKey
            .APP_VERSION_INFO])
        }).catchError((errorResponse) => { throw errorResponse });
    }
    return info;
  }
  static Future<String> signUpCall(String apiKey, UserDetails userDetails) async {
    String response;
    if (apiKey != null && apiKey.isNotEmpty)
      response = await getClient(apiKey).signUp(userDetails);
    return response;
  }
  static Future<String> fetchLegalEntities(String apiKey) async {
    String legalEntities;
    if (apiKey != null && apiKey.isNotEmpty)
      legalEntities = await getClient(apiKey).fetchLegalEntities();
    return legalEntities;
  }
  static Future<String> fetchBusinessCategories(String apiKey) async {
    String businessCategories;
    if (apiKey != null && apiKey.isNotEmpty) {
      businessCategories = await getClient(apiKey).fetchBusinessCategories();
    }
    return businessCategories;
  }
  static Future<FormData> prepareFormData(File file, String fieldKey, String businessId) async {
    return FormData.fromMap({
      JsonFieldKey.BUSINESS_ID: businessId,
      fieldKey: await MultipartFile.fromFile(
        file.path,
        filename: file.path.split("/").last,
      ),
    });
  }
  static Future<String> fetchOfficePremises(String apiKey) async {
    String officePremises;
    if (apiKey != null && apiKey.isNotEmpty)
      officePremises = await getClient(apiKey).fetchOfficePremises();
    return officePremises;
  }
  static Future<bool> saveBusinessId(String key, String value) async {
    final prefs = await SharedPreferences.getInstance();
    return prefs.setString(key, json.encode(value));
  }
  static Future<String> getBusinessId(String key) async {
    final prefs = await SharedPreferences.getInstance();
    return prefs.getString(key);
  }
  static Future<bool> removeBusinessId(String key) async {
    final prefs = await SharedPreferences.getInstance();
    return prefs.remove(key);
  }
  static Future<String> fetchUserDetails(String apiKey,
      UserDetailsRequest request) async {
    String userDetails;
    if (apiKey != null && apiKey.isNotEmpty)
      userDetails = await getClient(apiKey).fetchUserDetails(request);
    return userDetails;
  }
  static Future<List<Bank>> fetchBanks(String apiKey) async {
    List<Bank> banks;
    if (apiKey != null && apiKey.isNotEmpty)
      banks = await getClient(apiKey).fetchBanks();
    return banks;
  }
  static Widget showImage(String field, String filePath) {
    if (filePath != null && filePath.isNotEmpty) {
      if (filePath.contains("https"))
        return Image.network(filePath);
      else
        return _showImageFromFile(filePath);
    } else return null;
  }
  static Widget _showImageFromFile(String filePath) {
    if (filePath != null) {
      if (filePath.isNotEmpty) {
        return Image.file(new File(filePath), fit: BoxFit.fill);
      }
    }
    return null;
  }
  static showLoaderDialogWithText(BuildContext context, String data) {
    AlertDialog alert = AlertDialog(
      content: Row(
        children: [
          CircularProgressIndicator(),
          Container(
            margin: EdgeInsets.only(left: 10.0, right: 10.0),
            child:Text(data)
          ),
        ]
      ),
    );
    showDialog(barrierDismissible: false,
      context:context,
      builder:(BuildContext context) {
        return alert;
      },
    );
  }
  static showSnackBar(final ScaffoldState _scaffoldState, String textData) {
    _scaffoldState.showSnackBar(SnackBar(content: Text(textData)));
  }
  static String showEnterFieldError(String fieldName) {
    return "Please enter $fieldName";
  }
  static Future<bool> showPreview(BuildContext context, File file, bool val)
  async {
    return Navigator.push(context, MaterialPageRoute(builder: (context) =>
      PreviewImage(
        file: file,
        isPreviewAgain: val,
      )
    ));
  }
  static void apiCallsHandler(BuildContext context, ScaffoldState currentState,
      Function function) {
    try {
      function();
    } on DioError catch(e) {
      Navigator.of(context).pop(false);
      if (e.type == DioErrorType.DEFAULT) {
        Utils.showSnackBar(currentState, GeneralError.CHECK_INTERNET_CONNECTION);
      }
    } on TimeoutException catch(_) {
      Navigator.of(context).pop(false);
      Utils.showSnackBar(currentState, GeneralError.REQUEST_TIMEOUT);
    } catch(e) {
      Navigator.of(context).pop(false);
    }
  }
  static Future<ImageResponse> upload(String apiKey, File file, String fieldId,
      String businessId) async {
    try {
      FormData formData = await prepareFormData(file, fieldId, businessId);
      final jsonData = await getClient(apiKey)
          .uploadImageToServer(formData)
          .timeout(
          Duration(seconds: Utils.timeOutSec),
          onTimeout: () {
            print("request timeout");
            return;
          }
      );
      if (jsonData != null && jsonData.isNotEmpty)
        return ImageResponse.fromJson(json.decode(jsonData), fieldId);
    } on DioError catch (e) {
      if (e.type == DioErrorType.DEFAULT) {
        return ImageResponse(
            responseCode: 1000,
            responseData: null,
            responseMessage: GeneralError.CHECK_INTERNET_CONNECTION
        );
      } else if (e.type == DioErrorType.RESPONSE) {
        if (e.error.toString().contains("[502]")) {
          return ImageResponse(
              responseCode: 1000,
              responseData: null,
              responseMessage: GeneralError.SERVICE_UNAVAILABLE
          );
        } else if (e.error.toString().contains("[404]")) {
          return ImageResponse(
              responseCode: 1000,
              responseData: null,
              responseMessage: GeneralError.SERVICE_UNAVAILABLE
          );
        }
      }
    } on TimeoutException catch (_) {
      return ImageResponse(
          responseCode: 1000,
          responseData: null,
          responseMessage: GeneralError.SERVICE_UNAVAILABLE
      );
    }
    return null;
  }
  static bool isFileExtensionSupported(File file) {
    String fileName = file.path.split("/").last;
    String extension = fileName.split(".").last;
    for (String val in General.ALLOWED_FILE_EXTENSIONS) {
      if (val == extension)
        return true;
    }
    return false;
  }
  static bool isGreaterThen5MB(File file) {
    if (file.lengthSync() > 5242880) return true;
    else return false;
  }
  static bool isNotGreaterThen5MB(File file) {
    if (file.lengthSync() < 5242880) return true;
    else return false;
  }
  static String showPartnerDirectorNameText(int number) {
    return "Name of Partner/Director $number";
  }
  static String showPartnerDirectorCnicText(int number) {
    return "CNIC of Partner/Director $number";
  }
  static Future<List<City>> fetchCities(String apiKey) async {
    return await getClient(apiKey).fetchCities();
  }
  static Future<List<Province>> fetchProvinces(String apiKey) async {
    return await getClient(apiKey).fetchProvinces();
  }
  static Widget showWithDottedBorder(Widget child) {
    return DottedBorder(
      color: Colors.grey,
      strokeWidth: 1,
      dashPattern: [6, 3],
      strokeCap: StrokeCap.round,
      padding: const EdgeInsets.all(Layout.dp5),
      child: child
    );
  }
  static Widget showTitleAndDescription(String title, String description) {
    return Expanded(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Padding(
            padding: const EdgeInsets.only(top: 5.0, left: 5.0),
            child: Align(
              alignment: Alignment.topLeft,
              child: Text(
                title,
                softWrap: true,
                style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: 17.0
                ),
              ),
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(top: 5.0, left: 5.0),
            child: Text(
              description,
              softWrap: true,
              style: TextStyle(
                fontSize: 12.0
              ),
            ),
          ),
        ],
      )
    );
  }
  static Widget uploadIcon() {
    return Center(
      child: Opacity(
        opacity: 0.5,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Icon(
              Icons.cloud_upload,
              size: 50.0,
            ),
            Text(General.UPLOAD)
          ],
        ),
      ),
    );
  }
  static Future<List<InstrumentType>> fetchInstrumentTypes(String apiKey) {
    return getClient(apiKey).fetchInstrumentTypes();
  }
  static Future<List<FeeBorne>> fetchFeeBorne(String apiKey) {
    return getClient(apiKey).fetchFeeBorne();
  }
  static Widget getLogoMark() {
    return Container(
      width: 150.0,
      child: Image.asset(AssetImage("assets/images/ic_foree_logo_mark.png")
        .assetName, package: 'businessonboarding'
      )
    );
  }
  static Widget getHelplineMark(){
    return IconButton(
          icon: new Icon(Icons.phone, size: 25.0),
          onPressed: (){
              launch("tel:+9221111136733");
            }
      );
  }
  static Widget getDeleteIcon() {
    return Container(
      width: 40,
      height: 40,
      child: Image.asset(AssetImage("assets/images/ic_delete.png")
        .assetName, package: 'businessonboarding',
      ),
    );
  }
  static Widget getAcceptIcon() {
    return Container(
      width: 40,
      height: 40,
      child: Image.asset(AssetImage("assets/images/ic_accept.png")
        .assetName, package: 'businessonboarding',
      ),
    );
  }
  static Widget getCancelIcon() {
    return Container(
      width: 40,
      height: 40,
      child: Image.asset(AssetImage("assets/images/ic_close.png")
        .assetName, package: 'businessonboarding',
      ),
    );
  }
  static Widget crossImage(Function function) {
    return Align(
      alignment: Alignment.topRight,
      child: GestureDetector(
        onTap: () => function(),
        child: Icon(Icons.cancel_outlined),
      ),
    );
  }
  static Future<User> acquireUser(String apiKey, AcquireUser acquireUser) {
    return getClient(apiKey).acquireUser(acquireUser);
  }
  static Future<User> updateUser(String apiKey, UpdateUser updateUser) {
    return getClient(apiKey).updateUser(updateUser);
  }
  static Future<Otp> requestOtp(String apiKey, OtpParams params, bool isEmailVisible) {
    if (isEmailVisible) return getClient(apiKey).sendOtpToEmail(params);
    else return getClient(apiKey).sendOtpToNumber(params);
  }
  static Future<User> createPassword(String apiKey, LoginOrCreatePasswordParams params) {
    return getClient(apiKey).createPassword(params);
  }
  static Future<User> login(String apiKey, LoginOrCreatePasswordParams params) {
    return getClient(apiKey).login(params);
  }
  static Future<User> verifyEmail(String apiKey, VerifyParams params) {
    return getClient(apiKey).verifyEmail(params);
  }
  static Future<User> verifyMobileNumber(String apiKey, VerifyParams params) {
    return getClient(apiKey).verifyPhone(params);
  }
  static Future<ForgotPassword> forgotPassword(String apiKey, ForgotPasswordParams params) {
    return getClient(apiKey).forgotPassword(params);
  }
  static Future<String> onBoardBusiness(String apiKey, BusinessDetailParams params) {
    return getClient(apiKey).onBoardBusiness(params);
  }
  static Future<String> fetchBusinessDetails(String apiKey, BusinessIdParams params) {
    return getClient(apiKey).fetchBusinessDetails(params);
  }
  static Future<String> fetchDocuments(String apiKey, BusinessIdParams params) {
    return getClient(apiKey).fetchDocuments(params);
  }
  static void showNetworkErrors(ScaffoldState state, var onError) {
    if (onError is DioError) {
      if (onError.type == DioErrorType.DEFAULT)
        Utils.showSnackBar(state, GeneralError.CHECK_INTERNET_CONNECTION);
      else Utils.showSnackBar(state, GeneralError.SERVICE_UNAVAILABLE);
    } else Utils.showSnackBar(state, GeneralError.SERVICE_UNAVAILABLE);
  }
}