import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import '../Data.dart';
import '../BusinessSignUpForm.dart';
import 'Constants.dart';
import 'PasswordScreen.dart';
import 'Requests/BusinessDetailParams.dart';
import 'Requests/OtpParams.dart';
import 'Requests/VerifyParams.dart';
import 'Responses/User.dart';
import 'Timer/bloc/timer_bloc.dart';
import 'Utilities/Utils.dart';
import 'Timer/ticker.dart';

class VerifyUser extends StatefulWidget {
  final String apiKey;
  final Map<String, String> data;
  final bool requiredEmailOTP;
  final Data preDefinedData;
  final User user;
  final BusinessDetailParams businessDetails;
  final ValueSetter<String> callbackBusinessId;
  VerifyUser({Key key, @required this.apiKey, this.data, this.requiredEmailOTP,
    this.preDefinedData, this.user, this.businessDetails,
    this.callbackBusinessId}) : super(key: key);

  @override
  _VerifyUser createState() => _VerifyUser();
}
class _VerifyUser extends State<VerifyUser> {
  final _formKey = GlobalKey<FormState>();
  // final _controller = TextEditingController();
  final _scaffoldStateKey = GlobalKey<ScaffoldState>();
  TimerBloc _timerBloc;
  Widget logo;
  // var otpController = TextEditingController();
  String _otp;
  String _data;

  @override
  void initState() {
    if (widget.requiredEmailOTP) {
      _data = widget.businessDetails.emailAddress;
      // _controller.text = widget.businessDetails.emailAddress;
    } else {
      _data = widget.businessDetails.mobileNumber;
      // _controller.text = widget.businessDetails.mobileNumber;
    }
    _timerBloc = TimerBloc(ticker: Ticker());
    _timerBloc.add(TimerStarted(duration: _timerBloc.state.duration));
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        child: Scaffold(
            key: _scaffoldStateKey,
            appBar: AppBar(
              title: Text(Titles.VERIFY_USER),
              leading: logo,
              automaticallyImplyLeading: Platform.isIOS,
            ),
            body: Padding(
              padding: PaddingDetails.FORM,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Expanded(
                    child: Form(
                      key: _formKey,
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          // Email or mobile
                          Padding(
                            padding: PaddingDetails.FIELD_TOP,
                            child: Stack(
                              alignment: Alignment.centerRight,
                              children: [
                                TextFormField(
                                  decoration: InputDecoration(
                                      border: const OutlineInputBorder(),
                                      labelText: (widget.requiredEmailOTP ? FormLabels.EMAIL_ADDRESS : FormLabels.MOBILE_NUMBER)
                                  ),
                                  // controller: _controller,
                                  enabled: false,
                                  initialValue: _data,
                                ),
                                IconButton(
                                  icon: Icon(Icons.edit),
                                  onPressed: _moveToSignUp,
                                )
                              ],
                            ),
                          ),
                          // OTP
                          Padding(
                            padding: PaddingDetails.FIELD_TOP,
                            child: Stack(
                              alignment: Alignment.centerRight,
                              children: [
                                TextFormField(
                                  // controller: otpController,
                                  onChanged: (value) => _otp = value,
                                  decoration: InputDecoration(
                                      border: const OutlineInputBorder(),
                                      labelText: FormLabels.OTP,
                                      suffixIcon: BlocProvider(
                                        create: (context) => _timerBloc,
                                        child: Padding(
                                          padding: PaddingDetails.LOGO_PADDING,
                                          child: Column(
                                            mainAxisAlignment: MainAxisAlignment.center,
                                            children: [
                                              BlocBuilder<TimerBloc, TimerState>(
                                                builder: (context, state) {
                                                  final String minutesStr = ((state.duration / 30) % 60)
                                                      .floor()
                                                      .toString()
                                                      .padLeft(2, '0');
                                                  final String secondsStr = (state.duration % 60)
                                                      .floor()
                                                      .toString()
                                                      .padLeft(2, '0');
                                                  if (minutesStr == "00" && secondsStr == "00") {
                                                    return RichText(
                                                        text: TextSpan(
                                                          style: TextStyle(color: Colors.blue),
                                                          children: <TextSpan>[
                                                            TextSpan(
                                                                text: TextOf.RESEND_OTP,
                                                                style: TextStyle(
                                                                    decoration: TextDecoration.underline
                                                                ),
                                                                recognizer: TapGestureRecognizer()
                                                                  ..onTap = () {
                                                                    // otpController.clear();
                                                                    _otp = "";
                                                                    Utils.showLoaderDialogWithText(context, "Resending OTP");
                                                                    _resendOtp();
                                                                  }
                                                            )
                                                          ],
                                                        )
                                                    );
                                                  } else {
                                                    return Text('$minutesStr:$secondsStr');
                                                  }
                                                },
                                              )
                                            ],
                                          ),
                                        ),
                                      )
                                  ),
                                  validator: (value) {
                                    if (value.isEmpty)
                                      return Utils.showEnterFieldError("OTP");
                                    else return null;
                                  },
                                  keyboardType: TextInputType.phone,
                                  inputFormatters: [
                                    LengthLimitingTextInputFormatter(6),
                                    FilteringTextInputFormatter.allow(RegExp("[0-9]"))
                                  ],
                                ),
                              ],
                            ),
                          ),
                          // Hint
                          Padding(
                            padding: PaddingDetails.HINT,
                            child: Text(TextOf.OTP + "$_data"),
                          ),
                        ],
                      ),
                    ),
                  ),
                  Container(
                    width: Layout.BTN_WIDTH,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: [
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            Utils.getHelplineMark(),
                            Utils.getLogoMark(),
                          ],
                        ),
                        SizedBox(
                          width: Layout.BTN_WIDTH,
                          child: ElevatedButton(
                              onPressed: () async {
                                print(_otp);
                                if (_formKey.currentState.validate()) {
                                  FocusScope.of(context).unfocus();
                                  Utils.showLoaderDialogWithText(context, "Verifying...");
                                  if (widget.requiredEmailOTP) {
                                    await Utils.verifyEmail(widget.apiKey,
                                        VerifyParams(
                                            token: _otp,// otpController.value.text,
                                            uId: widget.user.responseData.id
                                        )
                                    ).then((value) {
                                      print(_otp);
                                      print(value.responseCode);
                                      if (value != null) {
                                        if (value.responseCode == 0) {
                                          Navigator.of(context).pop(false);
                                          createPassword();
                                        } else {
                                          Navigator.of(context).pop(false);
                                          Utils.showSnackBar(_scaffoldStateKey
                                              .currentState,
                                              value.responseMessage);
                                        }
                                      }
                                    }).timeout(
                                        Duration(seconds: Utils.timeOutSec),
                                        onTimeout: () {
                                          Navigator.of(context).pop(false);
                                          Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
                                          return;
                                        }
                                    ).catchError((onError) {
                                      print(onError);
                                      Navigator.of(context).pop(false);
                                      Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
                                    });
                                  } else {
                                    await Utils.verifyMobileNumber(widget.apiKey,
                                        VerifyParams(
                                            token: _otp/*otpController.value.text*/,
                                            uId: widget.user.responseData.id
                                        )
                                    ).then((value) {
                                      if (value != null) {
                                        if (value.responseCode == 0) {
                                          Navigator.of(context).pop(false);
                                          createPassword();
                                        } else {
                                          Navigator.of(context).pop(false);
                                          Utils.showSnackBar(_scaffoldStateKey
                                              .currentState,
                                              value.responseMessage);
                                        }
                                      }
                                    }).timeout(
                                        Duration(seconds: Utils.timeOutSec),
                                        onTimeout: () {
                                          Navigator.of(context).pop(false);
                                          Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
                                          return;
                                        }
                                    ).catchError((onError) {
                                      print(onError);
                                      Navigator.of(context).pop(false);
                                      Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
                                    });
                                  }
                                }
                              },
                              child: Text(
                                ButtonTags.VERIFY,
                              )
                          ),
                        )
                      ],
                    ),
                  )
                ],
              ),
            )
        ),
        onWillPop: () {
          _moveToSignUp();
          return Future.value(false);
        }
    );
  }
  void _moveToSignUp() {
    Navigator.pushReplacement(context, MaterialPageRoute(
        builder: (context) => BusinessSignUpForm(
          apiKey: widget.apiKey,
          data: widget.data,
          predefinedData: widget.preDefinedData,
          user: widget.user,
          loginFromEmail: widget.requiredEmailOTP,
          callbackBusinessId: widget.callbackBusinessId,
        )
    )).then((value) {
      if (value != null && value[0] && value[1])
        Navigator.of(context).pop(value);
      else Navigator.of(context).pop([true, false]);
    });
  }
  void _resendOtp() {
    Utils.requestOtp(widget.apiKey,
        OtpParams(userId: widget.user.responseData.id),
        widget.requiredEmailOTP).then((response) {
      if (response.requiresCode == 0) {
        Navigator.of(context).pop(false);
        setState(() {
          _timerBloc.add(TimerStarted(duration: 60));
        });
      } else {
        Navigator.of(context).pop(false);
        Utils.showSnackBar(_scaffoldStateKey.currentState,
            response.responseData.errorMessage);
      }
    }).timeout(
        Duration(seconds: Utils.timeOutSec),
        onTimeout: () {
          Navigator.of(context).pop(false);
          Utils.showSnackBar(_scaffoldStateKey.currentState, GeneralError.SERVICE_UNAVAILABLE);
        }
    ).catchError((onError) {
      print(onError);
      Navigator.of(context).pop(false);
      Utils.showNetworkErrors(_scaffoldStateKey.currentState, onError);
    });
  }
  void createPassword() {
    Navigator.push(context, MaterialPageRoute(builder: (context) =>
        PasswordScreen(
          apiKey: widget.apiKey,
          data: widget.data,
          isLogin: false,
          user: widget.user,
          businessDetails: widget.businessDetails,
          callbackBusinessId: widget.callbackBusinessId,
        )
    )).then((value) {
      if (value != null && value[0] && !value[1])
        _moveToSignUp();
      else if (value != null && value[0] && value[1])
        Navigator.of(context).pop(value);
    });
  }
}